"""
Generated by generate/generate.py - 2020-08-02 10:03:59.857088
"""
from aiogithubapi.objects.repository.pullrequest import (
    AIOGitHubAPIRepositoryPullrequest,
)

from tests.legacy.responses.repository.pullrequest_fixtrue import (
    pullrequest_fixtrue_response,
)


def test_pullrequest(pullrequest_fixtrue_response):
    obj = AIOGitHubAPIRepositoryPullrequest(pullrequest_fixtrue_response)
    assert obj.url == pullrequest_fixtrue_response["url"]
    assert obj.id == pullrequest_fixtrue_response["id"]
    assert obj.node_id == pullrequest_fixtrue_response["node_id"]
    assert obj.html_url == pullrequest_fixtrue_response["html_url"]
    assert obj.diff_url == pullrequest_fixtrue_response["diff_url"]
    assert obj.patch_url == pullrequest_fixtrue_response["patch_url"]
    assert obj.issue_url == pullrequest_fixtrue_response["issue_url"]
    assert obj.commits_url == pullrequest_fixtrue_response["commits_url"]
    assert obj.review_comments_url == pullrequest_fixtrue_response["review_comments_url"]
    assert obj.review_comment_url == pullrequest_fixtrue_response["review_comment_url"]
    assert obj.comments_url == pullrequest_fixtrue_response["comments_url"]
    assert obj.statuses_url == pullrequest_fixtrue_response["statuses_url"]
    assert obj.number == pullrequest_fixtrue_response["number"]
    assert obj.state == pullrequest_fixtrue_response["state"]
    assert obj.locked == pullrequest_fixtrue_response["locked"]
    assert obj.title == pullrequest_fixtrue_response["title"]
    assert obj.user.login == pullrequest_fixtrue_response["user"]["login"]
    assert obj.user.id == pullrequest_fixtrue_response["user"]["id"]
    assert obj.user.node_id == pullrequest_fixtrue_response["user"]["node_id"]
    assert obj.user.avatar_url == pullrequest_fixtrue_response["user"]["avatar_url"]
    assert obj.user.gravatar_id == pullrequest_fixtrue_response["user"]["gravatar_id"]
    assert obj.user.url == pullrequest_fixtrue_response["user"]["url"]
    assert obj.user.html_url == pullrequest_fixtrue_response["user"]["html_url"]
    assert obj.user.followers_url == pullrequest_fixtrue_response["user"]["followers_url"]
    assert obj.user.following_url == pullrequest_fixtrue_response["user"]["following_url"]
    assert obj.user.gists_url == pullrequest_fixtrue_response["user"]["gists_url"]
    assert obj.user.starred_url == pullrequest_fixtrue_response["user"]["starred_url"]
    assert obj.user.subscriptions_url == pullrequest_fixtrue_response["user"]["subscriptions_url"]
    assert obj.user.organizations_url == pullrequest_fixtrue_response["user"]["organizations_url"]
    assert obj.user.repos_url == pullrequest_fixtrue_response["user"]["repos_url"]
    assert obj.user.events_url == pullrequest_fixtrue_response["user"]["events_url"]
    assert (
        obj.user.received_events_url == pullrequest_fixtrue_response["user"]["received_events_url"]
    )
    assert obj.user.type == pullrequest_fixtrue_response["user"]["type"]
    assert obj.user.site_admin == pullrequest_fixtrue_response["user"]["site_admin"]
    assert obj.body == pullrequest_fixtrue_response["body"]
    assert obj.labels[0].id == pullrequest_fixtrue_response["labels"][0]["id"]
    assert obj.labels[0].node_id == pullrequest_fixtrue_response["labels"][0]["node_id"]
    assert obj.labels[0].url == pullrequest_fixtrue_response["labels"][0]["url"]
    assert obj.labels[0].name == pullrequest_fixtrue_response["labels"][0]["name"]
    assert obj.labels[0].description == pullrequest_fixtrue_response["labels"][0]["description"]
    assert obj.labels[0].color == pullrequest_fixtrue_response["labels"][0]["color"]
    assert obj.labels[0].default == pullrequest_fixtrue_response["labels"][0]["default"]
    assert obj.milestone.url == pullrequest_fixtrue_response["milestone"]["url"]
    assert obj.milestone.html_url == pullrequest_fixtrue_response["milestone"]["html_url"]
    assert obj.milestone.labels_url == pullrequest_fixtrue_response["milestone"]["labels_url"]
    assert obj.milestone.id == pullrequest_fixtrue_response["milestone"]["id"]
    assert obj.milestone.node_id == pullrequest_fixtrue_response["milestone"]["node_id"]
    assert obj.milestone.number == pullrequest_fixtrue_response["milestone"]["number"]
    assert obj.milestone.state == pullrequest_fixtrue_response["milestone"]["state"]
    assert obj.milestone.title == pullrequest_fixtrue_response["milestone"]["title"]
    assert obj.milestone.description == pullrequest_fixtrue_response["milestone"]["description"]
    assert (
        obj.milestone.creator.login == pullrequest_fixtrue_response["milestone"]["creator"]["login"]
    )
    assert obj.milestone.creator.id == pullrequest_fixtrue_response["milestone"]["creator"]["id"]
    assert (
        obj.milestone.creator.node_id
        == pullrequest_fixtrue_response["milestone"]["creator"]["node_id"]
    )
    assert (
        obj.milestone.creator.avatar_url
        == pullrequest_fixtrue_response["milestone"]["creator"]["avatar_url"]
    )
    assert (
        obj.milestone.creator.gravatar_id
        == pullrequest_fixtrue_response["milestone"]["creator"]["gravatar_id"]
    )
    assert obj.milestone.creator.url == pullrequest_fixtrue_response["milestone"]["creator"]["url"]
    assert (
        obj.milestone.creator.html_url
        == pullrequest_fixtrue_response["milestone"]["creator"]["html_url"]
    )
    assert (
        obj.milestone.creator.followers_url
        == pullrequest_fixtrue_response["milestone"]["creator"]["followers_url"]
    )
    assert (
        obj.milestone.creator.following_url
        == pullrequest_fixtrue_response["milestone"]["creator"]["following_url"]
    )
    assert (
        obj.milestone.creator.gists_url
        == pullrequest_fixtrue_response["milestone"]["creator"]["gists_url"]
    )
    assert (
        obj.milestone.creator.starred_url
        == pullrequest_fixtrue_response["milestone"]["creator"]["starred_url"]
    )
    assert (
        obj.milestone.creator.subscriptions_url
        == pullrequest_fixtrue_response["milestone"]["creator"]["subscriptions_url"]
    )
    assert (
        obj.milestone.creator.organizations_url
        == pullrequest_fixtrue_response["milestone"]["creator"]["organizations_url"]
    )
    assert (
        obj.milestone.creator.repos_url
        == pullrequest_fixtrue_response["milestone"]["creator"]["repos_url"]
    )
    assert (
        obj.milestone.creator.events_url
        == pullrequest_fixtrue_response["milestone"]["creator"]["events_url"]
    )
    assert (
        obj.milestone.creator.received_events_url
        == pullrequest_fixtrue_response["milestone"]["creator"]["received_events_url"]
    )
    assert (
        obj.milestone.creator.type == pullrequest_fixtrue_response["milestone"]["creator"]["type"]
    )
    assert (
        obj.milestone.creator.site_admin
        == pullrequest_fixtrue_response["milestone"]["creator"]["site_admin"]
    )
    assert obj.milestone.open_issues == pullrequest_fixtrue_response["milestone"]["open_issues"]
    assert obj.milestone.closed_issues == pullrequest_fixtrue_response["milestone"]["closed_issues"]
    assert obj.milestone.created_at == pullrequest_fixtrue_response["milestone"]["created_at"]
    assert obj.milestone.updated_at == pullrequest_fixtrue_response["milestone"]["updated_at"]
    assert obj.milestone.closed_at == pullrequest_fixtrue_response["milestone"]["closed_at"]
    assert obj.milestone.due_on == pullrequest_fixtrue_response["milestone"]["due_on"]
    assert obj.active_lock_reason == pullrequest_fixtrue_response["active_lock_reason"]
    assert obj.created_at == pullrequest_fixtrue_response["created_at"]
    assert obj.updated_at == pullrequest_fixtrue_response["updated_at"]
    assert obj.closed_at == pullrequest_fixtrue_response["closed_at"]
    assert obj.merged_at == pullrequest_fixtrue_response["merged_at"]
    assert obj.merge_commit_sha == pullrequest_fixtrue_response["merge_commit_sha"]
    assert obj.assignee.login == pullrequest_fixtrue_response["assignee"]["login"]
    assert obj.assignee.id == pullrequest_fixtrue_response["assignee"]["id"]
    assert obj.assignee.node_id == pullrequest_fixtrue_response["assignee"]["node_id"]
    assert obj.assignee.avatar_url == pullrequest_fixtrue_response["assignee"]["avatar_url"]
    assert obj.assignee.gravatar_id == pullrequest_fixtrue_response["assignee"]["gravatar_id"]
    assert obj.assignee.url == pullrequest_fixtrue_response["assignee"]["url"]
    assert obj.assignee.html_url == pullrequest_fixtrue_response["assignee"]["html_url"]
    assert obj.assignee.followers_url == pullrequest_fixtrue_response["assignee"]["followers_url"]
    assert obj.assignee.following_url == pullrequest_fixtrue_response["assignee"]["following_url"]
    assert obj.assignee.gists_url == pullrequest_fixtrue_response["assignee"]["gists_url"]
    assert obj.assignee.starred_url == pullrequest_fixtrue_response["assignee"]["starred_url"]
    assert (
        obj.assignee.subscriptions_url
        == pullrequest_fixtrue_response["assignee"]["subscriptions_url"]
    )
    assert (
        obj.assignee.organizations_url
        == pullrequest_fixtrue_response["assignee"]["organizations_url"]
    )
    assert obj.assignee.repos_url == pullrequest_fixtrue_response["assignee"]["repos_url"]
    assert obj.assignee.events_url == pullrequest_fixtrue_response["assignee"]["events_url"]
    assert (
        obj.assignee.received_events_url
        == pullrequest_fixtrue_response["assignee"]["received_events_url"]
    )
    assert obj.assignee.type == pullrequest_fixtrue_response["assignee"]["type"]
    assert obj.assignee.site_admin == pullrequest_fixtrue_response["assignee"]["site_admin"]
    assert obj.assignees[0].login == pullrequest_fixtrue_response["assignees"][0]["login"]
    assert obj.assignees[0].id == pullrequest_fixtrue_response["assignees"][0]["id"]
    assert obj.assignees[0].node_id == pullrequest_fixtrue_response["assignees"][0]["node_id"]
    assert obj.assignees[0].avatar_url == pullrequest_fixtrue_response["assignees"][0]["avatar_url"]
    assert (
        obj.assignees[0].gravatar_id == pullrequest_fixtrue_response["assignees"][0]["gravatar_id"]
    )
    assert obj.assignees[0].url == pullrequest_fixtrue_response["assignees"][0]["url"]
    assert obj.assignees[0].html_url == pullrequest_fixtrue_response["assignees"][0]["html_url"]
    assert (
        obj.assignees[0].followers_url
        == pullrequest_fixtrue_response["assignees"][0]["followers_url"]
    )
    assert (
        obj.assignees[0].following_url
        == pullrequest_fixtrue_response["assignees"][0]["following_url"]
    )
    assert obj.assignees[0].gists_url == pullrequest_fixtrue_response["assignees"][0]["gists_url"]
    assert (
        obj.assignees[0].starred_url == pullrequest_fixtrue_response["assignees"][0]["starred_url"]
    )
    assert (
        obj.assignees[0].subscriptions_url
        == pullrequest_fixtrue_response["assignees"][0]["subscriptions_url"]
    )
    assert (
        obj.assignees[0].organizations_url
        == pullrequest_fixtrue_response["assignees"][0]["organizations_url"]
    )
    assert obj.assignees[0].repos_url == pullrequest_fixtrue_response["assignees"][0]["repos_url"]
    assert obj.assignees[0].events_url == pullrequest_fixtrue_response["assignees"][0]["events_url"]
    assert (
        obj.assignees[0].received_events_url
        == pullrequest_fixtrue_response["assignees"][0]["received_events_url"]
    )
    assert obj.assignees[0].type == pullrequest_fixtrue_response["assignees"][0]["type"]
    assert obj.assignees[0].site_admin == pullrequest_fixtrue_response["assignees"][0]["site_admin"]
    assert (
        obj.requested_reviewers[0].login
        == pullrequest_fixtrue_response["requested_reviewers"][0]["login"]
    )
    assert (
        obj.requested_reviewers[0].id
        == pullrequest_fixtrue_response["requested_reviewers"][0]["id"]
    )
    assert (
        obj.requested_reviewers[0].node_id
        == pullrequest_fixtrue_response["requested_reviewers"][0]["node_id"]
    )
    assert (
        obj.requested_reviewers[0].avatar_url
        == pullrequest_fixtrue_response["requested_reviewers"][0]["avatar_url"]
    )
    assert (
        obj.requested_reviewers[0].gravatar_id
        == pullrequest_fixtrue_response["requested_reviewers"][0]["gravatar_id"]
    )
    assert (
        obj.requested_reviewers[0].url
        == pullrequest_fixtrue_response["requested_reviewers"][0]["url"]
    )
    assert (
        obj.requested_reviewers[0].html_url
        == pullrequest_fixtrue_response["requested_reviewers"][0]["html_url"]
    )
    assert (
        obj.requested_reviewers[0].followers_url
        == pullrequest_fixtrue_response["requested_reviewers"][0]["followers_url"]
    )
    assert (
        obj.requested_reviewers[0].following_url
        == pullrequest_fixtrue_response["requested_reviewers"][0]["following_url"]
    )
    assert (
        obj.requested_reviewers[0].gists_url
        == pullrequest_fixtrue_response["requested_reviewers"][0]["gists_url"]
    )
    assert (
        obj.requested_reviewers[0].starred_url
        == pullrequest_fixtrue_response["requested_reviewers"][0]["starred_url"]
    )
    assert (
        obj.requested_reviewers[0].subscriptions_url
        == pullrequest_fixtrue_response["requested_reviewers"][0]["subscriptions_url"]
    )
    assert (
        obj.requested_reviewers[0].organizations_url
        == pullrequest_fixtrue_response["requested_reviewers"][0]["organizations_url"]
    )
    assert (
        obj.requested_reviewers[0].repos_url
        == pullrequest_fixtrue_response["requested_reviewers"][0]["repos_url"]
    )
    assert (
        obj.requested_reviewers[0].events_url
        == pullrequest_fixtrue_response["requested_reviewers"][0]["events_url"]
    )
    assert (
        obj.requested_reviewers[0].received_events_url
        == pullrequest_fixtrue_response["requested_reviewers"][0]["received_events_url"]
    )
    assert (
        obj.requested_reviewers[0].type
        == pullrequest_fixtrue_response["requested_reviewers"][0]["type"]
    )
    assert (
        obj.requested_reviewers[0].site_admin
        == pullrequest_fixtrue_response["requested_reviewers"][0]["site_admin"]
    )
    assert obj.requested_teams[0].id == pullrequest_fixtrue_response["requested_teams"][0]["id"]
    assert (
        obj.requested_teams[0].node_id
        == pullrequest_fixtrue_response["requested_teams"][0]["node_id"]
    )
    assert obj.requested_teams[0].url == pullrequest_fixtrue_response["requested_teams"][0]["url"]
    assert (
        obj.requested_teams[0].html_url
        == pullrequest_fixtrue_response["requested_teams"][0]["html_url"]
    )
    assert obj.requested_teams[0].name == pullrequest_fixtrue_response["requested_teams"][0]["name"]
    assert obj.requested_teams[0].slug == pullrequest_fixtrue_response["requested_teams"][0]["slug"]
    assert (
        obj.requested_teams[0].description
        == pullrequest_fixtrue_response["requested_teams"][0]["description"]
    )
    assert (
        obj.requested_teams[0].privacy
        == pullrequest_fixtrue_response["requested_teams"][0]["privacy"]
    )
    assert (
        obj.requested_teams[0].permission
        == pullrequest_fixtrue_response["requested_teams"][0]["permission"]
    )
    assert (
        obj.requested_teams[0].members_url
        == pullrequest_fixtrue_response["requested_teams"][0]["members_url"]
    )
    assert (
        obj.requested_teams[0].repositories_url
        == pullrequest_fixtrue_response["requested_teams"][0]["repositories_url"]
    )
    assert (
        obj.requested_teams[0].parent
        == pullrequest_fixtrue_response["requested_teams"][0]["parent"]
    )
    assert obj.head.label == pullrequest_fixtrue_response["head"]["label"]
    assert obj.head.ref == pullrequest_fixtrue_response["head"]["ref"]
    assert obj.head.sha == pullrequest_fixtrue_response["head"]["sha"]
    assert obj.head.user.login == pullrequest_fixtrue_response["head"]["user"]["login"]
    assert obj.head.user.id == pullrequest_fixtrue_response["head"]["user"]["id"]
    assert obj.head.user.node_id == pullrequest_fixtrue_response["head"]["user"]["node_id"]
    assert obj.head.user.avatar_url == pullrequest_fixtrue_response["head"]["user"]["avatar_url"]
    assert obj.head.user.gravatar_id == pullrequest_fixtrue_response["head"]["user"]["gravatar_id"]
    assert obj.head.user.url == pullrequest_fixtrue_response["head"]["user"]["url"]
    assert obj.head.user.html_url == pullrequest_fixtrue_response["head"]["user"]["html_url"]
    assert (
        obj.head.user.followers_url == pullrequest_fixtrue_response["head"]["user"]["followers_url"]
    )
    assert (
        obj.head.user.following_url == pullrequest_fixtrue_response["head"]["user"]["following_url"]
    )
    assert obj.head.user.gists_url == pullrequest_fixtrue_response["head"]["user"]["gists_url"]
    assert obj.head.user.starred_url == pullrequest_fixtrue_response["head"]["user"]["starred_url"]
    assert (
        obj.head.user.subscriptions_url
        == pullrequest_fixtrue_response["head"]["user"]["subscriptions_url"]
    )
    assert (
        obj.head.user.organizations_url
        == pullrequest_fixtrue_response["head"]["user"]["organizations_url"]
    )
    assert obj.head.user.repos_url == pullrequest_fixtrue_response["head"]["user"]["repos_url"]
    assert obj.head.user.events_url == pullrequest_fixtrue_response["head"]["user"]["events_url"]
    assert (
        obj.head.user.received_events_url
        == pullrequest_fixtrue_response["head"]["user"]["received_events_url"]
    )
    assert obj.head.user.type == pullrequest_fixtrue_response["head"]["user"]["type"]
    assert obj.head.user.site_admin == pullrequest_fixtrue_response["head"]["user"]["site_admin"]
    assert obj.head.repo.id == pullrequest_fixtrue_response["head"]["repo"]["id"]
    assert obj.head.repo.node_id == pullrequest_fixtrue_response["head"]["repo"]["node_id"]
    assert obj.head.repo.name == pullrequest_fixtrue_response["head"]["repo"]["name"]
    assert obj.head.repo.full_name == pullrequest_fixtrue_response["head"]["repo"]["full_name"]
    assert (
        obj.head.repo.owner.login == pullrequest_fixtrue_response["head"]["repo"]["owner"]["login"]
    )
    assert obj.head.repo.owner.id == pullrequest_fixtrue_response["head"]["repo"]["owner"]["id"]
    assert (
        obj.head.repo.owner.node_id
        == pullrequest_fixtrue_response["head"]["repo"]["owner"]["node_id"]
    )
    assert (
        obj.head.repo.owner.avatar_url
        == pullrequest_fixtrue_response["head"]["repo"]["owner"]["avatar_url"]
    )
    assert (
        obj.head.repo.owner.gravatar_id
        == pullrequest_fixtrue_response["head"]["repo"]["owner"]["gravatar_id"]
    )
    assert obj.head.repo.owner.url == pullrequest_fixtrue_response["head"]["repo"]["owner"]["url"]
    assert (
        obj.head.repo.owner.html_url
        == pullrequest_fixtrue_response["head"]["repo"]["owner"]["html_url"]
    )
    assert (
        obj.head.repo.owner.followers_url
        == pullrequest_fixtrue_response["head"]["repo"]["owner"]["followers_url"]
    )
    assert (
        obj.head.repo.owner.following_url
        == pullrequest_fixtrue_response["head"]["repo"]["owner"]["following_url"]
    )
    assert (
        obj.head.repo.owner.gists_url
        == pullrequest_fixtrue_response["head"]["repo"]["owner"]["gists_url"]
    )
    assert (
        obj.head.repo.owner.starred_url
        == pullrequest_fixtrue_response["head"]["repo"]["owner"]["starred_url"]
    )
    assert (
        obj.head.repo.owner.subscriptions_url
        == pullrequest_fixtrue_response["head"]["repo"]["owner"]["subscriptions_url"]
    )
    assert (
        obj.head.repo.owner.organizations_url
        == pullrequest_fixtrue_response["head"]["repo"]["owner"]["organizations_url"]
    )
    assert (
        obj.head.repo.owner.repos_url
        == pullrequest_fixtrue_response["head"]["repo"]["owner"]["repos_url"]
    )
    assert (
        obj.head.repo.owner.events_url
        == pullrequest_fixtrue_response["head"]["repo"]["owner"]["events_url"]
    )
    assert (
        obj.head.repo.owner.received_events_url
        == pullrequest_fixtrue_response["head"]["repo"]["owner"]["received_events_url"]
    )
    assert obj.head.repo.owner.type == pullrequest_fixtrue_response["head"]["repo"]["owner"]["type"]
    assert (
        obj.head.repo.owner.site_admin
        == pullrequest_fixtrue_response["head"]["repo"]["owner"]["site_admin"]
    )
    assert obj.head.repo.private == pullrequest_fixtrue_response["head"]["repo"]["private"]
    assert obj.head.repo.html_url == pullrequest_fixtrue_response["head"]["repo"]["html_url"]
    assert obj.head.repo.description == pullrequest_fixtrue_response["head"]["repo"]["description"]
    assert obj.head.repo.fork == pullrequest_fixtrue_response["head"]["repo"]["fork"]
    assert obj.head.repo.url == pullrequest_fixtrue_response["head"]["repo"]["url"]
    assert obj.head.repo.archive_url == pullrequest_fixtrue_response["head"]["repo"]["archive_url"]
    assert (
        obj.head.repo.assignees_url == pullrequest_fixtrue_response["head"]["repo"]["assignees_url"]
    )
    assert obj.head.repo.blobs_url == pullrequest_fixtrue_response["head"]["repo"]["blobs_url"]
    assert (
        obj.head.repo.branches_url == pullrequest_fixtrue_response["head"]["repo"]["branches_url"]
    )
    assert (
        obj.head.repo.collaborators_url
        == pullrequest_fixtrue_response["head"]["repo"]["collaborators_url"]
    )
    assert (
        obj.head.repo.comments_url == pullrequest_fixtrue_response["head"]["repo"]["comments_url"]
    )
    assert obj.head.repo.commits_url == pullrequest_fixtrue_response["head"]["repo"]["commits_url"]
    assert obj.head.repo.compare_url == pullrequest_fixtrue_response["head"]["repo"]["compare_url"]
    assert (
        obj.head.repo.contents_url == pullrequest_fixtrue_response["head"]["repo"]["contents_url"]
    )
    assert (
        obj.head.repo.contributors_url
        == pullrequest_fixtrue_response["head"]["repo"]["contributors_url"]
    )
    assert (
        obj.head.repo.deployments_url
        == pullrequest_fixtrue_response["head"]["repo"]["deployments_url"]
    )
    assert (
        obj.head.repo.downloads_url == pullrequest_fixtrue_response["head"]["repo"]["downloads_url"]
    )
    assert obj.head.repo.events_url == pullrequest_fixtrue_response["head"]["repo"]["events_url"]
    assert obj.head.repo.forks_url == pullrequest_fixtrue_response["head"]["repo"]["forks_url"]
    assert (
        obj.head.repo.git_commits_url
        == pullrequest_fixtrue_response["head"]["repo"]["git_commits_url"]
    )
    assert (
        obj.head.repo.git_refs_url == pullrequest_fixtrue_response["head"]["repo"]["git_refs_url"]
    )
    assert (
        obj.head.repo.git_tags_url == pullrequest_fixtrue_response["head"]["repo"]["git_tags_url"]
    )
    assert obj.head.repo.git_url == pullrequest_fixtrue_response["head"]["repo"]["git_url"]
    assert (
        obj.head.repo.issue_comment_url
        == pullrequest_fixtrue_response["head"]["repo"]["issue_comment_url"]
    )
    assert (
        obj.head.repo.issue_events_url
        == pullrequest_fixtrue_response["head"]["repo"]["issue_events_url"]
    )
    assert obj.head.repo.issues_url == pullrequest_fixtrue_response["head"]["repo"]["issues_url"]
    assert obj.head.repo.keys_url == pullrequest_fixtrue_response["head"]["repo"]["keys_url"]
    assert obj.head.repo.labels_url == pullrequest_fixtrue_response["head"]["repo"]["labels_url"]
    assert (
        obj.head.repo.languages_url == pullrequest_fixtrue_response["head"]["repo"]["languages_url"]
    )
    assert obj.head.repo.merges_url == pullrequest_fixtrue_response["head"]["repo"]["merges_url"]
    assert (
        obj.head.repo.milestones_url
        == pullrequest_fixtrue_response["head"]["repo"]["milestones_url"]
    )
    assert (
        obj.head.repo.notifications_url
        == pullrequest_fixtrue_response["head"]["repo"]["notifications_url"]
    )
    assert obj.head.repo.pulls_url == pullrequest_fixtrue_response["head"]["repo"]["pulls_url"]
    assert (
        obj.head.repo.releases_url == pullrequest_fixtrue_response["head"]["repo"]["releases_url"]
    )
    assert obj.head.repo.ssh_url == pullrequest_fixtrue_response["head"]["repo"]["ssh_url"]
    assert (
        obj.head.repo.stargazers_url
        == pullrequest_fixtrue_response["head"]["repo"]["stargazers_url"]
    )
    assert (
        obj.head.repo.statuses_url == pullrequest_fixtrue_response["head"]["repo"]["statuses_url"]
    )
    assert (
        obj.head.repo.subscribers_url
        == pullrequest_fixtrue_response["head"]["repo"]["subscribers_url"]
    )
    assert (
        obj.head.repo.subscription_url
        == pullrequest_fixtrue_response["head"]["repo"]["subscription_url"]
    )
    assert obj.head.repo.tags_url == pullrequest_fixtrue_response["head"]["repo"]["tags_url"]
    assert obj.head.repo.teams_url == pullrequest_fixtrue_response["head"]["repo"]["teams_url"]
    assert obj.head.repo.trees_url == pullrequest_fixtrue_response["head"]["repo"]["trees_url"]
    assert obj.head.repo.clone_url == pullrequest_fixtrue_response["head"]["repo"]["clone_url"]
    assert obj.head.repo.mirror_url == pullrequest_fixtrue_response["head"]["repo"]["mirror_url"]
    assert obj.head.repo.hooks_url == pullrequest_fixtrue_response["head"]["repo"]["hooks_url"]
    assert obj.head.repo.svn_url == pullrequest_fixtrue_response["head"]["repo"]["svn_url"]
    assert obj.head.repo.homepage == pullrequest_fixtrue_response["head"]["repo"]["homepage"]
    assert obj.head.repo.language == pullrequest_fixtrue_response["head"]["repo"]["language"]
    assert obj.head.repo.forks_count == pullrequest_fixtrue_response["head"]["repo"]["forks_count"]
    assert (
        obj.head.repo.stargazers_count
        == pullrequest_fixtrue_response["head"]["repo"]["stargazers_count"]
    )
    assert (
        obj.head.repo.watchers_count
        == pullrequest_fixtrue_response["head"]["repo"]["watchers_count"]
    )
    assert obj.head.repo.size == pullrequest_fixtrue_response["head"]["repo"]["size"]
    assert (
        obj.head.repo.default_branch
        == pullrequest_fixtrue_response["head"]["repo"]["default_branch"]
    )
    assert (
        obj.head.repo.open_issues_count
        == pullrequest_fixtrue_response["head"]["repo"]["open_issues_count"]
    )
    assert obj.head.repo.is_template == pullrequest_fixtrue_response["head"]["repo"]["is_template"]
    assert obj.head.repo.topics[0] == pullrequest_fixtrue_response["head"]["repo"]["topics"][0]
    assert obj.head.repo.has_issues == pullrequest_fixtrue_response["head"]["repo"]["has_issues"]
    assert (
        obj.head.repo.has_projects == pullrequest_fixtrue_response["head"]["repo"]["has_projects"]
    )
    assert obj.head.repo.has_wiki == pullrequest_fixtrue_response["head"]["repo"]["has_wiki"]
    assert obj.head.repo.has_pages == pullrequest_fixtrue_response["head"]["repo"]["has_pages"]
    assert (
        obj.head.repo.has_downloads == pullrequest_fixtrue_response["head"]["repo"]["has_downloads"]
    )
    assert obj.head.repo.archived == pullrequest_fixtrue_response["head"]["repo"]["archived"]
    assert obj.head.repo.disabled == pullrequest_fixtrue_response["head"]["repo"]["disabled"]
    assert obj.head.repo.visibility == pullrequest_fixtrue_response["head"]["repo"]["visibility"]
    assert obj.head.repo.pushed_at == pullrequest_fixtrue_response["head"]["repo"]["pushed_at"]
    assert obj.head.repo.created_at == pullrequest_fixtrue_response["head"]["repo"]["created_at"]
    assert obj.head.repo.updated_at == pullrequest_fixtrue_response["head"]["repo"]["updated_at"]
    assert (
        obj.head.repo.permissions.admin
        == pullrequest_fixtrue_response["head"]["repo"]["permissions"]["admin"]
    )
    assert (
        obj.head.repo.permissions.push
        == pullrequest_fixtrue_response["head"]["repo"]["permissions"]["push"]
    )
    assert (
        obj.head.repo.permissions.pull
        == pullrequest_fixtrue_response["head"]["repo"]["permissions"]["pull"]
    )
    assert (
        obj.head.repo.allow_rebase_merge
        == pullrequest_fixtrue_response["head"]["repo"]["allow_rebase_merge"]
    )
    assert (
        obj.head.repo.template_repository
        == pullrequest_fixtrue_response["head"]["repo"]["template_repository"]
    )
    assert (
        obj.head.repo.temp_clone_token
        == pullrequest_fixtrue_response["head"]["repo"]["temp_clone_token"]
    )
    assert (
        obj.head.repo.allow_squash_merge
        == pullrequest_fixtrue_response["head"]["repo"]["allow_squash_merge"]
    )
    assert (
        obj.head.repo.delete_branch_on_merge
        == pullrequest_fixtrue_response["head"]["repo"]["delete_branch_on_merge"]
    )
    assert (
        obj.head.repo.allow_merge_commit
        == pullrequest_fixtrue_response["head"]["repo"]["allow_merge_commit"]
    )
    assert (
        obj.head.repo.subscribers_count
        == pullrequest_fixtrue_response["head"]["repo"]["subscribers_count"]
    )
    assert (
        obj.head.repo.network_count == pullrequest_fixtrue_response["head"]["repo"]["network_count"]
    )
    assert obj.base.label == pullrequest_fixtrue_response["base"]["label"]
    assert obj.base.ref == pullrequest_fixtrue_response["base"]["ref"]
    assert obj.base.sha == pullrequest_fixtrue_response["base"]["sha"]
    assert obj.base.user.login == pullrequest_fixtrue_response["base"]["user"]["login"]
    assert obj.base.user.id == pullrequest_fixtrue_response["base"]["user"]["id"]
    assert obj.base.user.node_id == pullrequest_fixtrue_response["base"]["user"]["node_id"]
    assert obj.base.user.avatar_url == pullrequest_fixtrue_response["base"]["user"]["avatar_url"]
    assert obj.base.user.gravatar_id == pullrequest_fixtrue_response["base"]["user"]["gravatar_id"]
    assert obj.base.user.url == pullrequest_fixtrue_response["base"]["user"]["url"]
    assert obj.base.user.html_url == pullrequest_fixtrue_response["base"]["user"]["html_url"]
    assert (
        obj.base.user.followers_url == pullrequest_fixtrue_response["base"]["user"]["followers_url"]
    )
    assert (
        obj.base.user.following_url == pullrequest_fixtrue_response["base"]["user"]["following_url"]
    )
    assert obj.base.user.gists_url == pullrequest_fixtrue_response["base"]["user"]["gists_url"]
    assert obj.base.user.starred_url == pullrequest_fixtrue_response["base"]["user"]["starred_url"]
    assert (
        obj.base.user.subscriptions_url
        == pullrequest_fixtrue_response["base"]["user"]["subscriptions_url"]
    )
    assert (
        obj.base.user.organizations_url
        == pullrequest_fixtrue_response["base"]["user"]["organizations_url"]
    )
    assert obj.base.user.repos_url == pullrequest_fixtrue_response["base"]["user"]["repos_url"]
    assert obj.base.user.events_url == pullrequest_fixtrue_response["base"]["user"]["events_url"]
    assert (
        obj.base.user.received_events_url
        == pullrequest_fixtrue_response["base"]["user"]["received_events_url"]
    )
    assert obj.base.user.type == pullrequest_fixtrue_response["base"]["user"]["type"]
    assert obj.base.user.site_admin == pullrequest_fixtrue_response["base"]["user"]["site_admin"]
    assert obj.base.repo.id == pullrequest_fixtrue_response["base"]["repo"]["id"]
    assert obj.base.repo.node_id == pullrequest_fixtrue_response["base"]["repo"]["node_id"]
    assert obj.base.repo.name == pullrequest_fixtrue_response["base"]["repo"]["name"]
    assert obj.base.repo.full_name == pullrequest_fixtrue_response["base"]["repo"]["full_name"]
    assert (
        obj.base.repo.owner.login == pullrequest_fixtrue_response["base"]["repo"]["owner"]["login"]
    )
    assert obj.base.repo.owner.id == pullrequest_fixtrue_response["base"]["repo"]["owner"]["id"]
    assert (
        obj.base.repo.owner.node_id
        == pullrequest_fixtrue_response["base"]["repo"]["owner"]["node_id"]
    )
    assert (
        obj.base.repo.owner.avatar_url
        == pullrequest_fixtrue_response["base"]["repo"]["owner"]["avatar_url"]
    )
    assert (
        obj.base.repo.owner.gravatar_id
        == pullrequest_fixtrue_response["base"]["repo"]["owner"]["gravatar_id"]
    )
    assert obj.base.repo.owner.url == pullrequest_fixtrue_response["base"]["repo"]["owner"]["url"]
    assert (
        obj.base.repo.owner.html_url
        == pullrequest_fixtrue_response["base"]["repo"]["owner"]["html_url"]
    )
    assert (
        obj.base.repo.owner.followers_url
        == pullrequest_fixtrue_response["base"]["repo"]["owner"]["followers_url"]
    )
    assert (
        obj.base.repo.owner.following_url
        == pullrequest_fixtrue_response["base"]["repo"]["owner"]["following_url"]
    )
    assert (
        obj.base.repo.owner.gists_url
        == pullrequest_fixtrue_response["base"]["repo"]["owner"]["gists_url"]
    )
    assert (
        obj.base.repo.owner.starred_url
        == pullrequest_fixtrue_response["base"]["repo"]["owner"]["starred_url"]
    )
    assert (
        obj.base.repo.owner.subscriptions_url
        == pullrequest_fixtrue_response["base"]["repo"]["owner"]["subscriptions_url"]
    )
    assert (
        obj.base.repo.owner.organizations_url
        == pullrequest_fixtrue_response["base"]["repo"]["owner"]["organizations_url"]
    )
    assert (
        obj.base.repo.owner.repos_url
        == pullrequest_fixtrue_response["base"]["repo"]["owner"]["repos_url"]
    )
    assert (
        obj.base.repo.owner.events_url
        == pullrequest_fixtrue_response["base"]["repo"]["owner"]["events_url"]
    )
    assert (
        obj.base.repo.owner.received_events_url
        == pullrequest_fixtrue_response["base"]["repo"]["owner"]["received_events_url"]
    )
    assert obj.base.repo.owner.type == pullrequest_fixtrue_response["base"]["repo"]["owner"]["type"]
    assert (
        obj.base.repo.owner.site_admin
        == pullrequest_fixtrue_response["base"]["repo"]["owner"]["site_admin"]
    )
    assert obj.base.repo.private == pullrequest_fixtrue_response["base"]["repo"]["private"]
    assert obj.base.repo.html_url == pullrequest_fixtrue_response["base"]["repo"]["html_url"]
    assert obj.base.repo.description == pullrequest_fixtrue_response["base"]["repo"]["description"]
    assert obj.base.repo.fork == pullrequest_fixtrue_response["base"]["repo"]["fork"]
    assert obj.base.repo.url == pullrequest_fixtrue_response["base"]["repo"]["url"]
    assert obj.base.repo.archive_url == pullrequest_fixtrue_response["base"]["repo"]["archive_url"]
    assert (
        obj.base.repo.assignees_url == pullrequest_fixtrue_response["base"]["repo"]["assignees_url"]
    )
    assert obj.base.repo.blobs_url == pullrequest_fixtrue_response["base"]["repo"]["blobs_url"]
    assert (
        obj.base.repo.branches_url == pullrequest_fixtrue_response["base"]["repo"]["branches_url"]
    )
    assert (
        obj.base.repo.collaborators_url
        == pullrequest_fixtrue_response["base"]["repo"]["collaborators_url"]
    )
    assert (
        obj.base.repo.comments_url == pullrequest_fixtrue_response["base"]["repo"]["comments_url"]
    )
    assert obj.base.repo.commits_url == pullrequest_fixtrue_response["base"]["repo"]["commits_url"]
    assert obj.base.repo.compare_url == pullrequest_fixtrue_response["base"]["repo"]["compare_url"]
    assert (
        obj.base.repo.contents_url == pullrequest_fixtrue_response["base"]["repo"]["contents_url"]
    )
    assert (
        obj.base.repo.contributors_url
        == pullrequest_fixtrue_response["base"]["repo"]["contributors_url"]
    )
    assert (
        obj.base.repo.deployments_url
        == pullrequest_fixtrue_response["base"]["repo"]["deployments_url"]
    )
    assert (
        obj.base.repo.downloads_url == pullrequest_fixtrue_response["base"]["repo"]["downloads_url"]
    )
    assert obj.base.repo.events_url == pullrequest_fixtrue_response["base"]["repo"]["events_url"]
    assert obj.base.repo.forks_url == pullrequest_fixtrue_response["base"]["repo"]["forks_url"]
    assert (
        obj.base.repo.git_commits_url
        == pullrequest_fixtrue_response["base"]["repo"]["git_commits_url"]
    )
    assert (
        obj.base.repo.git_refs_url == pullrequest_fixtrue_response["base"]["repo"]["git_refs_url"]
    )
    assert (
        obj.base.repo.git_tags_url == pullrequest_fixtrue_response["base"]["repo"]["git_tags_url"]
    )
    assert obj.base.repo.git_url == pullrequest_fixtrue_response["base"]["repo"]["git_url"]
    assert (
        obj.base.repo.issue_comment_url
        == pullrequest_fixtrue_response["base"]["repo"]["issue_comment_url"]
    )
    assert (
        obj.base.repo.issue_events_url
        == pullrequest_fixtrue_response["base"]["repo"]["issue_events_url"]
    )
    assert obj.base.repo.issues_url == pullrequest_fixtrue_response["base"]["repo"]["issues_url"]
    assert obj.base.repo.keys_url == pullrequest_fixtrue_response["base"]["repo"]["keys_url"]
    assert obj.base.repo.labels_url == pullrequest_fixtrue_response["base"]["repo"]["labels_url"]
    assert (
        obj.base.repo.languages_url == pullrequest_fixtrue_response["base"]["repo"]["languages_url"]
    )
    assert obj.base.repo.merges_url == pullrequest_fixtrue_response["base"]["repo"]["merges_url"]
    assert (
        obj.base.repo.milestones_url
        == pullrequest_fixtrue_response["base"]["repo"]["milestones_url"]
    )
    assert (
        obj.base.repo.notifications_url
        == pullrequest_fixtrue_response["base"]["repo"]["notifications_url"]
    )
    assert obj.base.repo.pulls_url == pullrequest_fixtrue_response["base"]["repo"]["pulls_url"]
    assert (
        obj.base.repo.releases_url == pullrequest_fixtrue_response["base"]["repo"]["releases_url"]
    )
    assert obj.base.repo.ssh_url == pullrequest_fixtrue_response["base"]["repo"]["ssh_url"]
    assert (
        obj.base.repo.stargazers_url
        == pullrequest_fixtrue_response["base"]["repo"]["stargazers_url"]
    )
    assert (
        obj.base.repo.statuses_url == pullrequest_fixtrue_response["base"]["repo"]["statuses_url"]
    )
    assert (
        obj.base.repo.subscribers_url
        == pullrequest_fixtrue_response["base"]["repo"]["subscribers_url"]
    )
    assert (
        obj.base.repo.subscription_url
        == pullrequest_fixtrue_response["base"]["repo"]["subscription_url"]
    )
    assert obj.base.repo.tags_url == pullrequest_fixtrue_response["base"]["repo"]["tags_url"]
    assert obj.base.repo.teams_url == pullrequest_fixtrue_response["base"]["repo"]["teams_url"]
    assert obj.base.repo.trees_url == pullrequest_fixtrue_response["base"]["repo"]["trees_url"]
    assert obj.base.repo.clone_url == pullrequest_fixtrue_response["base"]["repo"]["clone_url"]
    assert obj.base.repo.mirror_url == pullrequest_fixtrue_response["base"]["repo"]["mirror_url"]
    assert obj.base.repo.hooks_url == pullrequest_fixtrue_response["base"]["repo"]["hooks_url"]
    assert obj.base.repo.svn_url == pullrequest_fixtrue_response["base"]["repo"]["svn_url"]
    assert obj.base.repo.homepage == pullrequest_fixtrue_response["base"]["repo"]["homepage"]
    assert obj.base.repo.language == pullrequest_fixtrue_response["base"]["repo"]["language"]
    assert obj.base.repo.forks_count == pullrequest_fixtrue_response["base"]["repo"]["forks_count"]
    assert (
        obj.base.repo.stargazers_count
        == pullrequest_fixtrue_response["base"]["repo"]["stargazers_count"]
    )
    assert (
        obj.base.repo.watchers_count
        == pullrequest_fixtrue_response["base"]["repo"]["watchers_count"]
    )
    assert obj.base.repo.size == pullrequest_fixtrue_response["base"]["repo"]["size"]
    assert (
        obj.base.repo.default_branch
        == pullrequest_fixtrue_response["base"]["repo"]["default_branch"]
    )
    assert (
        obj.base.repo.open_issues_count
        == pullrequest_fixtrue_response["base"]["repo"]["open_issues_count"]
    )
    assert obj.base.repo.is_template == pullrequest_fixtrue_response["base"]["repo"]["is_template"]
    assert obj.base.repo.topics[0] == pullrequest_fixtrue_response["base"]["repo"]["topics"][0]
    assert obj.base.repo.has_issues == pullrequest_fixtrue_response["base"]["repo"]["has_issues"]
    assert (
        obj.base.repo.has_projects == pullrequest_fixtrue_response["base"]["repo"]["has_projects"]
    )
    assert obj.base.repo.has_wiki == pullrequest_fixtrue_response["base"]["repo"]["has_wiki"]
    assert obj.base.repo.has_pages == pullrequest_fixtrue_response["base"]["repo"]["has_pages"]
    assert (
        obj.base.repo.has_downloads == pullrequest_fixtrue_response["base"]["repo"]["has_downloads"]
    )
    assert obj.base.repo.archived == pullrequest_fixtrue_response["base"]["repo"]["archived"]
    assert obj.base.repo.disabled == pullrequest_fixtrue_response["base"]["repo"]["disabled"]
    assert obj.base.repo.visibility == pullrequest_fixtrue_response["base"]["repo"]["visibility"]
    assert obj.base.repo.pushed_at == pullrequest_fixtrue_response["base"]["repo"]["pushed_at"]
    assert obj.base.repo.created_at == pullrequest_fixtrue_response["base"]["repo"]["created_at"]
    assert obj.base.repo.updated_at == pullrequest_fixtrue_response["base"]["repo"]["updated_at"]
    assert (
        obj.base.repo.permissions.admin
        == pullrequest_fixtrue_response["base"]["repo"]["permissions"]["admin"]
    )
    assert (
        obj.base.repo.permissions.push
        == pullrequest_fixtrue_response["base"]["repo"]["permissions"]["push"]
    )
    assert (
        obj.base.repo.permissions.pull
        == pullrequest_fixtrue_response["base"]["repo"]["permissions"]["pull"]
    )
    assert (
        obj.base.repo.allow_rebase_merge
        == pullrequest_fixtrue_response["base"]["repo"]["allow_rebase_merge"]
    )
    assert (
        obj.base.repo.template_repository
        == pullrequest_fixtrue_response["base"]["repo"]["template_repository"]
    )
    assert (
        obj.base.repo.temp_clone_token
        == pullrequest_fixtrue_response["base"]["repo"]["temp_clone_token"]
    )
    assert (
        obj.base.repo.allow_squash_merge
        == pullrequest_fixtrue_response["base"]["repo"]["allow_squash_merge"]
    )
    assert (
        obj.base.repo.delete_branch_on_merge
        == pullrequest_fixtrue_response["base"]["repo"]["delete_branch_on_merge"]
    )
    assert (
        obj.base.repo.allow_merge_commit
        == pullrequest_fixtrue_response["base"]["repo"]["allow_merge_commit"]
    )
    assert (
        obj.base.repo.subscribers_count
        == pullrequest_fixtrue_response["base"]["repo"]["subscribers_count"]
    )
    assert (
        obj.base.repo.network_count == pullrequest_fixtrue_response["base"]["repo"]["network_count"]
    )
    assert obj.author_association == pullrequest_fixtrue_response["author_association"]
    assert obj.draft == pullrequest_fixtrue_response["draft"]
    assert obj.merged == pullrequest_fixtrue_response["merged"]
    assert obj.mergeable == pullrequest_fixtrue_response["mergeable"]
    assert obj.rebaseable == pullrequest_fixtrue_response["rebaseable"]
    assert obj.mergeable_state == pullrequest_fixtrue_response["mergeable_state"]
    assert obj.merged_by.login == pullrequest_fixtrue_response["merged_by"]["login"]
    assert obj.merged_by.id == pullrequest_fixtrue_response["merged_by"]["id"]
    assert obj.merged_by.node_id == pullrequest_fixtrue_response["merged_by"]["node_id"]
    assert obj.merged_by.avatar_url == pullrequest_fixtrue_response["merged_by"]["avatar_url"]
    assert obj.merged_by.gravatar_id == pullrequest_fixtrue_response["merged_by"]["gravatar_id"]
    assert obj.merged_by.url == pullrequest_fixtrue_response["merged_by"]["url"]
    assert obj.merged_by.html_url == pullrequest_fixtrue_response["merged_by"]["html_url"]
    assert obj.merged_by.followers_url == pullrequest_fixtrue_response["merged_by"]["followers_url"]
    assert obj.merged_by.following_url == pullrequest_fixtrue_response["merged_by"]["following_url"]
    assert obj.merged_by.gists_url == pullrequest_fixtrue_response["merged_by"]["gists_url"]
    assert obj.merged_by.starred_url == pullrequest_fixtrue_response["merged_by"]["starred_url"]
    assert (
        obj.merged_by.subscriptions_url
        == pullrequest_fixtrue_response["merged_by"]["subscriptions_url"]
    )
    assert (
        obj.merged_by.organizations_url
        == pullrequest_fixtrue_response["merged_by"]["organizations_url"]
    )
    assert obj.merged_by.repos_url == pullrequest_fixtrue_response["merged_by"]["repos_url"]
    assert obj.merged_by.events_url == pullrequest_fixtrue_response["merged_by"]["events_url"]
    assert (
        obj.merged_by.received_events_url
        == pullrequest_fixtrue_response["merged_by"]["received_events_url"]
    )
    assert obj.merged_by.type == pullrequest_fixtrue_response["merged_by"]["type"]
    assert obj.merged_by.site_admin == pullrequest_fixtrue_response["merged_by"]["site_admin"]
    assert obj.comments == pullrequest_fixtrue_response["comments"]
    assert obj.review_comments == pullrequest_fixtrue_response["review_comments"]
    assert obj.maintainer_can_modify == pullrequest_fixtrue_response["maintainer_can_modify"]
    assert obj.commits == pullrequest_fixtrue_response["commits"]
    assert obj.additions == pullrequest_fixtrue_response["additions"]
    assert obj.deletions == pullrequest_fixtrue_response["deletions"]
    assert obj.changed_files == pullrequest_fixtrue_response["changed_files"]



"""Fixtures."""
# pylint: disable=missing-docstring
import pytest


@pytest.fixture()
def releases_response():
    return [
        {
            "url": "https://api.github.com/repos/octocat/Hello-World/releases/1",
            "html_url": "https://github.com/octocat/Hello-World/releases/v1.0.0",
            "assets_url": "https://api.github.com/repos/octocat/Hello-World/releases/1/assets",
            "upload_url": "https://uploads.github.com/repos/octocat/Hello-World/releases/1/assets{?name,label}",
            "tarball_url": "https://api.github.com/repos/octocat/Hello-World/tarball/v1.0.0",
            "zipball_url": "https://api.github.com/repos/octocat/Hello-World/zipball/v1.0.0",
            "id": 1,
            "node_id": "MDc6UmVsZWFzZTE=",
            "tag_name": "v1.0.0",
            "target_commitish": "main",
            "name": "v1.0.0",
            "body": "Description of the release",
            "draft": False,
            "prerelease": False,
            "created_at": "2013-02-27T19:35:32Z",
            "published_at": "2013-02-27T19:35:32Z",
            "author": {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
            "assets": [
                {
                    "url": "https://api.github.com/repos/octocat/Hello-World/releases/assets/1",
                    "browser_download_url": "https://github.com/octocat/Hello-World/releases/download/v1.0.0/example.zip",
                    "id": 1,
                    "node_id": "MDEyOlJlbGVhc2VBc3NldDE=",
                    "name": "example.zip",
                    "label": "short description",
                    "state": "uploaded",
                    "content_type": "application/zip",
                    "size": 1024,
                    "download_count": 42,
                    "created_at": "2013-02-27T19:35:32Z",
                    "updated_at": "2013-02-27T19:35:32Z",
                    "uploader": {
                        "login": "octocat",
                        "id": 1,
                        "node_id": "MDQ6VXNlcjE=",
                        "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                        "gravatar_id": "",
                        "url": "https://api.github.com/users/octocat",
                        "html_url": "https://github.com/octocat",
                        "followers_url": "https://api.github.com/users/octocat/followers",
                        "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                        "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                        "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                        "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                        "organizations_url": "https://api.github.com/users/octocat/orgs",
                        "repos_url": "https://api.github.com/users/octocat/repos",
                        "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                        "received_events_url": "https://api.github.com/users/octocat/received_events",
                        "type": "User",
                        "site_admin": False,
                    },
                }
            ],
        },
        {
            "url": "https://api.github.com/repos/octocat/Hello-World/releases/1",
            "html_url": "https://github.com/octocat/Hello-World/releases/v1.0.0",
            "assets_url": "https://api.github.com/repos/octocat/Hello-World/releases/1/assets",
            "upload_url": "https://uploads.github.com/repos/octocat/Hello-World/releases/1/assets{?name,label}",
            "tarball_url": "https://api.github.com/repos/octocat/Hello-World/tarball/v1.0.0",
            "zipball_url": "https://api.github.com/repos/octocat/Hello-World/zipball/v1.0.0",
            "id": 1,
            "node_id": "MDc6UmVsZWFzZTE=",
            "tag_name": "v1.0.0",
            "target_commitish": "main",
            "name": "v1.0.0",
            "body": "Description of the release",
            "draft": False,
            "prerelease": True,
            "created_at": "2013-02-27T19:35:32Z",
            "published_at": "2013-02-27T19:35:32Z",
            "author": {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
            "assets": [
                {
                    "url": "https://api.github.com/repos/octocat/Hello-World/releases/assets/1",
                    "browser_download_url": "https://github.com/octocat/Hello-World/releases/download/v1.0.0/example.zip",
                    "id": 1,
                    "node_id": "MDEyOlJlbGVhc2VBc3NldDE=",
                    "name": "example.zip",
                    "label": "short description",
                    "state": "uploaded",
                    "content_type": "application/zip",
                    "size": 1024,
                    "download_count": 42,
                    "created_at": "2013-02-27T19:35:32Z",
                    "updated_at": "2013-02-27T19:35:32Z",
                    "uploader": {
                        "login": "octocat",
                        "id": 1,
                        "node_id": "MDQ6VXNlcjE=",
                        "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                        "gravatar_id": "",
                        "url": "https://api.github.com/users/octocat",
                        "html_url": "https://github.com/octocat",
                        "followers_url": "https://api.github.com/users/octocat/followers",
                        "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                        "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                        "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                        "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                        "organizations_url": "https://api.github.com/users/octocat/orgs",
                        "repos_url": "https://api.github.com/users/octocat/repos",
                        "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                        "received_events_url": "https://api.github.com/users/octocat/received_events",
                        "type": "User",
                        "site_admin": False,
                    },
                }
            ],
        },
    ]



"""GitHub release data class."""

from __future__ import annotations

from typing import Any, Dict

from .base import GitHubDataModelBase
from .reaction import GitHubReactionModel
from .user import GitHubBaseUserModel


class GitHubReleaseAssetModel(GitHubDataModelBase):
    """Representation of a GitHub release asset."""

    url: str | None = None
    id: int | None = None
    name: str | None = None
    label: str | None = None
    uploader: GitHubBaseUserModel | None = None
    content_type: str | None = None
    state: str | None = None
    size: int | None = None
    download_count: int = None
    created_at: str = None
    updated_at: str = None
    browser_download_url: str = None

    def _generate_uploader(self, data: Dict[str, Any] | None) -> GitHubBaseUserModel:
        """Generate uploader data."""
        return GitHubBaseUserModel(data) if data else None


class GitHubReleaseModel(GitHubDataModelBase):
    """GitHub release data class."""

    url: str | None = None
    assets_url: str | None = None
    upload_url: str | None = None
    html_url: str | None = None
    id: int | None = None
    discussion_url: str | None = None
    author: GitHubBaseUserModel | None = None
    tag_name: str | None = None
    target_commitish: str | None = None
    name: str | None = None
    draft: bool | None = None
    prerelease: bool | None = None
    created_at: str | None = None
    published_at: str | None = None
    assets: list[dict] | None = None
    tarball_url: str | None = None
    zipball_url: str | None = None
    body: str | None = None
    reactions: GitHubReactionModel | None = None
    mentions_count: int | None = None

    def _generate_author(self, data: dict) -> GitHubBaseUserModel:
        """Generate author data."""
        return GitHubBaseUserModel(data)

    def _generate_assets(self, data: list[Dict[str, Any]]) -> dict:
        """Generate assets data."""
        return [GitHubReleaseAssetModel(asset) for asset in data or []]

    def _generate_reactions(self, data: dict) -> GitHubReactionModel:
        """Generate reactions data."""
        return GitHubReactionModel(data)



"""
Generated by generate/generate.py - 2020-08-02 10:33:09.248066
"""
from aiogithubapi.objects.orgs.team import AIOGitHubAPIOrgsTeam

from tests.legacy.responses.orgs.team_fixtrue import team_fixtrue_response


def test_team(team_fixtrue_response):
    obj = AIOGitHubAPIOrgsTeam(team_fixtrue_response)
    assert obj.id == team_fixtrue_response["id"]
    assert obj.node_id == team_fixtrue_response["node_id"]
    assert obj.url == team_fixtrue_response["url"]
    assert obj.html_url == team_fixtrue_response["html_url"]
    assert obj.name == team_fixtrue_response["name"]
    assert obj.slug == team_fixtrue_response["slug"]
    assert obj.description == team_fixtrue_response["description"]
    assert obj.privacy == team_fixtrue_response["privacy"]
    assert obj.permission == team_fixtrue_response["permission"]
    assert obj.members_url == team_fixtrue_response["members_url"]
    assert obj.repositories_url == team_fixtrue_response["repositories_url"]
    assert obj.parent == team_fixtrue_response["parent"]
    assert obj.members_count == team_fixtrue_response["members_count"]
    assert obj.repos_count == team_fixtrue_response["repos_count"]
    assert obj.created_at == team_fixtrue_response["created_at"]
    assert obj.updated_at == team_fixtrue_response["updated_at"]
    assert obj.organization.login == team_fixtrue_response["organization"]["login"]
    assert obj.organization.id == team_fixtrue_response["organization"]["id"]
    assert obj.organization.node_id == team_fixtrue_response["organization"]["node_id"]
    assert obj.organization.url == team_fixtrue_response["organization"]["url"]
    assert obj.organization.repos_url == team_fixtrue_response["organization"]["repos_url"]
    assert obj.organization.events_url == team_fixtrue_response["organization"]["events_url"]
    assert obj.organization.hooks_url == team_fixtrue_response["organization"]["hooks_url"]
    assert obj.organization.issues_url == team_fixtrue_response["organization"]["issues_url"]
    assert obj.organization.members_url == team_fixtrue_response["organization"]["members_url"]
    assert (
        obj.organization.public_members_url
        == team_fixtrue_response["organization"]["public_members_url"]
    )
    assert obj.organization.avatar_url == team_fixtrue_response["organization"]["avatar_url"]
    assert obj.organization.description == team_fixtrue_response["organization"]["description"]
    assert obj.organization.name == team_fixtrue_response["organization"]["name"]
    assert obj.organization.company == team_fixtrue_response["organization"]["company"]
    assert obj.organization.blog == team_fixtrue_response["organization"]["blog"]
    assert obj.organization.location == team_fixtrue_response["organization"]["location"]
    assert obj.organization.email == team_fixtrue_response["organization"]["email"]
    assert (
        obj.organization.twitter_username
        == team_fixtrue_response["organization"]["twitter_username"]
    )
    assert obj.organization.is_verified == team_fixtrue_response["organization"]["is_verified"]
    assert (
        obj.organization.has_organization_projects
        == team_fixtrue_response["organization"]["has_organization_projects"]
    )
    assert (
        obj.organization.has_repository_projects
        == team_fixtrue_response["organization"]["has_repository_projects"]
    )
    assert obj.organization.public_repos == team_fixtrue_response["organization"]["public_repos"]
    assert obj.organization.public_gists == team_fixtrue_response["organization"]["public_gists"]
    assert obj.organization.followers == team_fixtrue_response["organization"]["followers"]
    assert obj.organization.following == team_fixtrue_response["organization"]["following"]
    assert obj.organization.html_url == team_fixtrue_response["organization"]["html_url"]
    assert obj.organization.created_at == team_fixtrue_response["organization"]["created_at"]
    assert obj.organization.type == team_fixtrue_response["organization"]["type"]



"""
Generated by generate/generate.py - 2020-08-02 10:35:28.920747
"""
import pytest


@pytest.fixture()
def user_fixtrue_response():
    return {
        "login": "octocat",
        "id": 1,
        "node_id": "MDQ6VXNlcjE=",
        "avatar_url": "https://github.com/images/error/octocat_happy.gif",
        "gravatar_id": "",
        "url": "https://api.github.com/users/octocat",
        "html_url": "https://github.com/octocat",
        "followers_url": "https://api.github.com/users/octocat/followers",
        "following_url": "https://api.github.com/users/octocat/following{/other_user}",
        "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
        "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
        "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
        "organizations_url": "https://api.github.com/users/octocat/orgs",
        "repos_url": "https://api.github.com/users/octocat/repos",
        "events_url": "https://api.github.com/users/octocat/events{/privacy}",
        "received_events_url": "https://api.github.com/users/octocat/received_events",
        "type": "User",
        "site_admin": False,
        "name": "monalisa octocat",
        "company": "GitHub",
        "blog": "https://github.com/blog",
        "location": "San Francisco",
        "email": "octocat@github.com",
        "hireable": False,
        "bio": "There once was...",
        "twitter_username": "monatheoctocat",
        "public_repos": 2,
        "public_gists": 1,
        "followers": 20,
        "following": 0,
        "created_at": "2008-01-14T04:33:35Z",
        "updated_at": "2008-01-14T04:33:35Z",
    }



"""GitHub login oauth data class."""

from __future__ import annotations

from .base import GitHubDataModelBase


class GitHubLoginOauthModel(GitHubDataModelBase):
    """GitHub login oauth data class."""

    access_token: str | None = None
    token_type: str | None = None
    scope: str | None = None



"""Fixtures."""
# pylint: disable=missing-docstring
import pytest


@pytest.fixture()
def contents_list_response():
    return [
        {
            "type": "file",
            "size": 625,
            "name": "octokit.rb",
            "path": "lib/octokit.rb",
            "sha": "fff6fe3a23bf1c8ea0692b4a883af99bee26fd3b",
            "url": "https://api.github.com/repos/octokit/octokit.rb/contents/lib/octokit.rb",
            "git_url": "https://api.github.com/repos/octokit/octokit.rb/git/blobs/fff6fe3a23bf1c8ea0692b4a883af99bee26fd3b",
            "html_url": "https://github.com/octokit/octokit.rb/blob/main/lib/octokit.rb",
            "download_url": "https://raw.githubusercontent.com/octokit/octokit.rb/main/lib/octokit.rb",
            "_links": {
                "self": "https://api.github.com/repos/octokit/octokit.rb/contents/lib/octokit.rb",
                "git": "https://api.github.com/repos/octokit/octokit.rb/git/blobs/fff6fe3a23bf1c8ea0692b4a883af99bee26fd3b",
                "html": "https://github.com/octokit/octokit.rb/blob/main/lib/octokit.rb",
            },
        },
        {
            "type": "dir",
            "size": 0,
            "name": "octokit",
            "path": "lib/octokit",
            "sha": "a84d88e7554fc1fa21bcbc4efae3c782a70d2b9d",
            "url": "https://api.github.com/repos/octokit/octokit.rb/contents/lib/octokit",
            "git_url": "https://api.github.com/repos/octokit/octokit.rb/git/trees/a84d88e7554fc1fa21bcbc4efae3c782a70d2b9d",
            "html_url": "https://github.com/octokit/octokit.rb/tree/main/lib/octokit",
            "download_url": None,
            "_links": {
                "self": "https://api.github.com/repos/octokit/octokit.rb/contents/lib/octokit",
                "git": "https://api.github.com/repos/octokit/octokit.rb/git/trees/a84d88e7554fc1fa21bcbc4efae3c782a70d2b9d",
                "html": "https://github.com/octokit/octokit.rb/tree/main/lib/octokit",
            },
        },
    ]


@pytest.fixture()
def contents_file_response():
    return {
        "type": "file",
        "encoding": "base64",
        "size": 5362,
        "name": "README.md",
        "path": "README.md",
        "content": "",
        "sha": "3d21ec53a331a6f037a91c368710b99387d012c1",
        "url": "https://api.github.com/repos/octokit/octokit.rb/contents/README.md",
        "git_url": "https://api.github.com/repos/octokit/octokit.rb/git/blobs/3d21ec53a331a6f037a91c368710b99387d012c1",
        "html_url": "https://github.com/octokit/octokit.rb/blob/main/README.md",
        "download_url": "https://raw.githubusercontent.com/octokit/octokit.rb/main/README.md",
        "_links": {
            "git": "https://api.github.com/repos/octokit/octokit.rb/git/blobs/3d21ec53a331a6f037a91c368710b99387d012c1",
            "self": "https://api.github.com/repos/octokit/octokit.rb/contents/README.md",
            "html": "https://github.com/octokit/octokit.rb/blob/main/README.md",
        },
    }



"""Fixtures."""
# pylint: disable=missing-docstring
import pytest


@pytest.fixture()
def rate_limit_response():
    return {
        "resources": {
            "core": {"limit": 5000, "remaining": 4999, "reset": 1372700873},
            "search": {"limit": 30, "remaining": 18, "reset": 1372697452},
            "graphql": {"limit": 5000, "remaining": 4993, "reset": 1372700389},
            "integration_manifest": {
                "limit": 5000,
                "remaining": 4999,
                "reset": 1551806725,
            },
        },
        "rate": {"limit": 5000, "remaining": 4999, "reset": 1372700873},
    }



"""Test git namespace."""
# pylint: disable=missing-docstring
import pytest

from aiogithubapi import GitHubAPI

from tests.common import TEST_REPOSITORY_NAME, MockedRequests


@pytest.mark.asyncio
async def test_get_tree(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.repos.git.get_tree(TEST_REPOSITORY_NAME, "master")
    assert response.status == 200
    assert response.data.tree[0].path == "README"
    assert mock_requests.called == 1
    assert (
        mock_requests.last_request["url"]
        == "https://api.github.com/repos/octocat/hello-world/git/trees/master"
    )



"""
AIOGitHubAPI: Repository Traffic

https://docs.github.com/en/rest/reference/repos#traffic
"""

from typing import Optional

from aiohttp.hdrs import IF_NONE_MATCH

from ..base import AIOGitHubAPIBaseClient
from ..repos.traffic.clones import AIOGitHubAPIReposTrafficClones
from ..repos.traffic.pageviews import AIOGitHubAPIReposTrafficPageviews


class AIOGitHubAPIRepositoryTraffic(AIOGitHubAPIBaseClient):
    """Repository Release GitHub API implementation."""

    @property
    def full_name(self) -> None:
        return self.attributes.get("full_name")

    async def get_views(self, etag: Optional[str] = None):
        _endpoint = f"/repos/{self.full_name}/traffic/views"
        _headers = {}
        if etag:
            _headers[IF_NONE_MATCH] = etag
        response = await self.client.get(endpoint=_endpoint, headers=_headers)
        return AIOGitHubAPIReposTrafficPageviews(response.data)

    async def get_clones(self, etag: Optional[str] = None) -> None:
        _endpoint = f"/repos/{self.full_name}/traffic/clones"
        _headers = {}
        if etag:
            _headers[IF_NONE_MATCH] = etag
        response = await self.client.get(endpoint=_endpoint, headers=_headers)
        return AIOGitHubAPIReposTrafficClones(response.data)



"""
Methods for the issues namespace

https://docs.github.com/en/rest/reference/issues
"""

from __future__ import annotations

from typing import Any, Dict, List

from ..const import (
    GitHubIssueLockReason,
    GitHubRequestKwarg,
    HttpMethod,
    RepositoryType,
)
from ..helpers import repository_full_name
from ..models.issue import GitHubIssueModel
from ..models.issue_comment import GitHubIssueCommentModel
from ..models.response import GitHubResponseModel
from .base import BaseNamespace


class GitHubIssuesNamespace(BaseNamespace):
    """
    Methods for the issues namespace

    Note: GitHub's REST API v3 considers every pull request an issue,
    but not every issue is a pull request. For this reason, "Issues"
    endpoints may return both issues and pull requests in the response.
    You can identify pull requests by the pull_request key.
    """

    async def get(
        self,
        repository: RepositoryType,
        issue_number: int | str,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubIssueModel]:
        """
         Get an issue

         **Arguments**:

         `repository`

         The repository the issue belong to, example "octocat/hello-world"

         `issue_number`

         The issue number to return, example "1"

        https://docs.github.com/en/rest/reference/issues#get-an-issue
        """
        response = await self._client.async_call_api(
            endpoint=f"/repos/{repository_full_name(repository)}/issues/{issue_number}",
            **kwargs,
        )
        response.data = GitHubIssueModel(response.data)
        return response

    async def create(
        self,
        repository: RepositoryType,
        data: Dict[str, Any],
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubIssueModel]:
        """
         Create an issue

         **Arguments**:

         `repository`

         The repository the issue belong to, example "octocat/hello-world"

         `data`

         The data the issue will be created with, example {"title": "new title"}

        https://docs.github.com/en/rest/reference/issues#create-an-issue
        """
        response = await self._client.async_call_api(
            endpoint=f"/repos/{repository_full_name(repository)}/issues",
            data=data,
            **{**kwargs, GitHubRequestKwarg.METHOD: HttpMethod.POST},
        )
        response.data = GitHubIssueModel(response.data)
        return response

    async def update(
        self,
        repository: RepositoryType,
        issue_number: int | str,
        data: Dict[str, Any],
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubIssueModel]:
        """
         Update an issue

         **Arguments**:

         `repository`

         The repository the issue belong to, example "octocat/hello-world"

         `issue_number`

         The issue number to update, example "1"

         `data`

         The data to update, example {"title": "new title"}

        https://docs.github.com/en/rest/reference/issues#update-an-issue
        """
        response = await self._client.async_call_api(
            endpoint=f"/repos/{repository_full_name(repository)}/issues/{issue_number}",
            data=data,
            **{**kwargs, GitHubRequestKwarg.METHOD: HttpMethod.PATCH},
        )
        response.data = GitHubIssueModel(response.data)
        return response

    async def lock(
        self,
        repository: RepositoryType,
        issue_number: int | str,
        lock_reason: GitHubIssueLockReason,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[None]:
        """
         Lock an issue

         **Arguments**:

         `repository`

         The repository the comment belong to, example "octocat/hello-world"

         `issue_number`

         The issue number to lock, example "1"

         `lock_reason`

         The reason why the issue was locked;
            * off-topic
            * too heated
            * resolved
            * spam

        https://docs.github.com/en/rest/reference/issues#lock-an-issue
        """
        response = await self._client.async_call_api(
            endpoint=(f"/repos/{repository_full_name(repository)}/issues/{issue_number}/lock"),
            data={"lock_reason": lock_reason},
            **{**kwargs, GitHubRequestKwarg.METHOD: HttpMethod.PUT},
        )
        return response

    async def unlock(
        self,
        repository: RepositoryType,
        issue_number: int | str,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[None]:
        """
         Unlock an issue

         **Arguments**:

         `repository`

         The repository the comment belong to, example "octocat/hello-world"

         `issue_number`

         The issue number to lock, example "1"

        https://docs.github.com/en/rest/reference/issues#unlock-an-issue
        """
        response = await self._client.async_call_api(
            endpoint=(f"/repos/{repository_full_name(repository)}/issues/{issue_number}/lock"),
            **{**kwargs, GitHubRequestKwarg.METHOD: HttpMethod.DELETE},
        )
        return response

    async def list(
        self,
        repository: RepositoryType,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[List[GitHubIssueModel]]:
        """
         List repository issues

         **Arguments**:

         `repository`

         The repository to return, example "octocat/hello-world"

        https://docs.github.com/en/rest/reference/issues#list-repository-issues
        """
        response = await self._client.async_call_api(
            endpoint=f"/repos/{repository_full_name(repository)}/issues",
            **kwargs,
        )
        response.data = [GitHubIssueModel(data) for data in response.data]
        return response

    async def get_comment(
        self,
        repository: RepositoryType,
        comment_id: int | str,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[List[GitHubIssueCommentModel]]:
        """
         Get an issue comment

         **Arguments**:

         `repository`

         The repository the issue belong to, example "octocat/hello-world"

         `comment_id`

         The ID of a comment in an issue number to return, example "1"

        https://docs.github.com/en/rest/reference/issues#list-issue-comments
        """
        response = await self._client.async_call_api(
            endpoint=(
                f"/repos/{repository_full_name(repository)}" f"/issues/comments/{comment_id}"
            ),
            **kwargs,
        )
        response.data = GitHubIssueCommentModel(response.data)
        return response

    async def create_comment(
        self,
        repository: RepositoryType,
        issue_number: int | str,
        data: Dict[str, Any],
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubIssueCommentModel]:
        """
         Create an issue comment

         **Arguments**:

         `repository`

         The repository the issue belong to, example "octocat/hello-world"

         `issue_number`

         The issue number to create a comment in, example "1"

         `data`

         The data the issue will be created with, example {"body": "test"}

        https://docs.github.com/en/rest/reference/issues#create-an-issue-comment
        """
        response = await self._client.async_call_api(
            endpoint=f"/repos/{repository_full_name(repository)}/issues/{issue_number}/comments",
            data=data,
            **{**kwargs, GitHubRequestKwarg.METHOD: HttpMethod.POST},
        )
        response.data = GitHubIssueCommentModel(response.data)
        return response

    async def update_comment(
        self,
        repository: RepositoryType,
        comment_id: int | str,
        data: Dict[str, Any],
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubIssueCommentModel]:
        """
         Update an issue comment

         **Arguments**:

         `repository`

         The repository the comment belong to, example "octocat/hello-world"

         `comment_id`

         The ID of a comment in an issue number to update, example "1"

         `data`

         The data the issue will be created with, example {"body": "test"}

        https://docs.github.com/en/rest/reference/issues#update-an-issue-comment
        """
        response = await self._client.async_call_api(
            endpoint=(
                f"/repos/{repository_full_name(repository)}" f"/issues/comments/{comment_id}"
            ),
            data=data,
            **{**kwargs, GitHubRequestKwarg.METHOD: HttpMethod.PATCH},
        )
        response.data = GitHubIssueCommentModel(response.data)
        return response

    async def delete_comment(
        self,
        repository: RepositoryType,
        comment_id: int | str,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[None]:
        """
         Delete an issue comment

         **Arguments**:

         `repository`

         The repository the comment belong to, example "octocat/hello-world"

         `comment_id`

         The ID of a comment in an issue number to delete, example "1"

        https://docs.github.com/en/rest/reference/issues#delete-an-issue-comment
        """
        response = await self._client.async_call_api(
            endpoint=(
                f"/repos/{repository_full_name(repository)}" f"/issues/comments/{comment_id}"
            ),
            **{**kwargs, GitHubRequestKwarg.METHOD: HttpMethod.DELETE},
        )
        return response

    async def list_comments(
        self,
        repository: RepositoryType,
        issue_number: int | str,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[List[GitHubIssueCommentModel]]:
        """
         List issue comments

         **Arguments**:

         `repository`

         The repository the comments belong to, example "octocat/hello-world"

         `issue_number`

         The issue number to return comments for, example "1"

        https://docs.github.com/en/rest/reference/issues#list-issue-comments
        """
        response = await self._client.async_call_api(
            endpoint=f"/repos/{repository_full_name(repository)}/issues/{issue_number}/comments",
            **kwargs,
        )
        response.data = [GitHubIssueCommentModel(data) for data in response.data]
        return response



"""Common tests helpers."""
from __future__ import annotations

from dataclasses import dataclass
import json
from logging import Logger
import os
from typing import Any, Dict

from yarl import URL

from aiogithubapi.const import BASE_API_HEADERS, LOGGER, HttpContentType

TEST_LOGGER: Logger = LOGGER

TOKEN = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
CLIENT_ID = "xxxxxx"
DEVICE_CODE = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"

TEST_ORGANIZATION = "octocat"
TEST_REPOSITORY_NAME = "octocat/hello-world"
TEST_USER_NAME = "octocat"

EXPECTED_ETAG = 'W/"1234567890abcdefghijklmnopqrstuvwxyz"'


def load_fixture(filename, asjson=False, legacy=True):
    """Load a fixture."""
    filename = f"{filename}.json" if "." not in filename else filename
    path = os.path.join(
        os.path.dirname(__file__),
        "legacy" if legacy else "",
        "fixtures",
        filename.lower().replace("/", "_"),
    )
    TEST_LOGGER.debug("Loading fixture from %s", path)
    with open(path, encoding="utf-8") as fptr:
        if asjson:
            return json.loads(fptr.read())
        return fptr.read()


TEST_REQUEST_HEADERS = {
    **BASE_API_HEADERS,
    "Authorization": f"token {TOKEN}",
}

HEADERS = {**load_fixture("headers.json", asjson=True, legacy=False)}
HEADERS_RATELIMITED = {
    **HEADERS,
    "X-RateLimit-Remaining": "0",
    "X-RateLimit-Used": "5000",
}
HEADERS_TEXT = {**HEADERS, "Content-Type": HttpContentType.TEXT_PLAIN}


TEXT_ENDPOINTS = (
    "markdown",
    "zen",
    "octocat",
    "repos/octocat/hello-world/readme/",
    "repos/octocat/hello-world/readme/test",
)
BYTES_ENDPOINT = ("tarball", "zipball")


@dataclass
class MockResponse:
    """Mock response class."""

    _count = 0

    throw_on_file_error = False

    mock_data: Any | None = None
    mock_data_list: list[Any] | None = None
    mock_endpoint: str = ""
    mock_headers: Dict[str, str] | None = None
    mock_raises: BaseException | None = None
    mock_status: int = 200

    @property
    def status(self):
        """status."""
        return self.mock_status

    @property
    def headers(self):
        """headers."""
        return self.mock_headers or HEADERS

    @property
    def links(self):
        """headers."""
        pages: Dict[str, dict] = {}
        if not self.headers.get("Link"):
            return pages
        for pageentry in self.headers["Link"].split(", "):
            link = URL(pageentry.split(";")[0].replace("<", "").replace(">", ""))
            rel_type = pageentry.split(";")[1].split("=")[1].replace('"', "")
            pages[rel_type] = {"rel": rel_type, "url": link}
        return pages

    async def json(self, **_):
        """json."""
        if self.mock_raises is not None:
            raise self.mock_raises  # pylint: disable=raising-bad-type
        if self.mock_data_list:
            data = self.mock_data_list[self._count]
            self._count += 1
            return data
        if self.mock_data is not None:
            return self.mock_data
        try:
            return load_fixture(self.mock_endpoint, asjson=True, legacy=False)
        except OSError:
            if self.throw_on_file_error:
                raise OSError(f"Missing fixture for {self.mock_endpoint}") from None
            return {}

    async def text(self, **_):
        """text."""
        if self.mock_raises is not None:
            raise self.mock_raises  # pylint: disable=raising-bad-type
        if self.mock_data_list:
            data = self.mock_data_list[self._count]
            self._count += 1
            return json.dumps(data)
        if self.mock_data is not None:
            return json.dumps(self.mock_data)
        try:
            file = (
                self.mock_endpoint
                if self.mock_endpoint not in TEXT_ENDPOINTS
                else f"{self.mock_endpoint}.txt"
            )
            return load_fixture(file, legacy=False)
        except OSError:
            if self.throw_on_file_error:
                raise OSError(f"Missing fixture for {self.mock_endpoint}") from None
            return ""

    async def read(self, **_):
        """read."""
        return b""

    def clear(self):
        """clear."""
        self.mock_data = None
        self.mock_endpoint = ""
        self.mock_headers = None
        self.mock_raises = None
        self.mock_status = 200


class MockedRequests:
    """Mock request class."""

    _calls = []

    def add(self, url: str):
        """add."""
        self._calls.append(url)

    def clear(self):
        """clear."""
        self._calls.clear()

    def __repr__(self) -> str:
        return f"<MockedRequests: {self._calls}>"

    @property
    def called(self) -> int:
        """count."""
        return len(self._calls)

    def has(self, string: str) -> bool:
        """has."""
        return bool([entry for entry in self._calls if string in entry])

    @property
    def last_request(self) -> MockResponse:
        """last url."""
        return self._calls[-1]



"""Constants for aiogithubapi."""

from __future__ import annotations

from dataclasses import dataclass
from enum import IntEnum, StrEnum
from logging import Logger, getLogger
from typing import Dict, Literal, TypeVar, Union

from aiohttp.hdrs import ACCEPT, CONTENT_TYPE, USER_AGENT

GenericType = TypeVar("GenericType")

PROJECT_VERSION = "main"
PROJECT_NAME = "aiogithubapi"
PROJECT_URL = "https://github.com/ludeeus/aiogithubapi"

# Deprecated, use `GitHubRequestAcceptHeader` instead
ACCEPT_HEADERS: Dict[str, str] = {
    "base": "application/vnd.github.v3.raw+json",
    "preview": "application/vnd.github.mercy-preview+json",
}

# This is the default user agent,
# but it is advised to use your own when building out your application
DEFAULT_USER_AGENT = f"aiogithubapi/{PROJECT_VERSION}"

# https://docs.github.com/en/rest/overview/api-versions
HEADER_GITHUB_API_VERSION = "X-GitHub-Api-Version"
DEFAULT_API_VERSION = "2022-11-28"

BASE_API_URL = "https://api.github.com"
BASE_GITHUB_URL = "https://github.com"
OAUTH_DEVICE_LOGIN_PATH = "/login/device/code"
OAUTH_ACCESS_TOKEN_PATH = "/login/oauth/access_token"
OAUTH_USER_LOGIN = "https://github.com/login/device"


LOGGER: Logger = getLogger(PROJECT_NAME)


@dataclass
class Repository:
    """Repository."""

    owner: str
    repo: str

    @property
    def full_name(self) -> str:
        """Full name."""
        return f"{self.owner}/{self.repo}"


RepositoryType = Union[str, Dict[Literal["owner", "repo"], str], Repository]


class GitHubRequestAcceptHeader(StrEnum):
    """
    GitHub uses vaious accept headers to enable certain features.

    https://docs.github.com/en/rest/overview/media-types
    """

    RAW_JSON = "application/vnd.github.v3.raw+json"
    BASE_JSON = "application/vnd.github.v3+json"
    PREVIEW_MERCY = "application/vnd.github.mercy-preview+json"

    BASE = RAW_JSON
    PREVIEW = PREVIEW_MERCY


class GitHubClientKwarg(StrEnum):
    """
    kwargs that are used by the client.

    These are used to construct the client and will affect all requests.

    HEADERS:
        Used to set the base headers for all requests.
    BASE_URL:
        Used to override the base url for all requests. Defaults to https://api.github.com .
    TIMEOUT:
        Used to set the timeout for all requests. Defaults to 20
    CLIENT_NAME:
        This name will be used as the user agent header.
    """

    HEADERS = "headers"
    BASE_URL = "base_url"
    TIMEOUT = "timeout"
    CLIENT_NAME = "client_name"


class GitHubRequestKwarg(StrEnum):
    """
    kwargs that are used by requests.

    ETAG:
        Used to set the IF_NONE_MATCH header, if this is set and the content
        is not modified GitHubNotModifiedException will be raised.
    HEADERS:
        Used to set the headers of the request.
    METHOD:
        Used to set the method of the request. Defaults to GET.
    PARAMS:
        Used to set the params of the request.
    QUERY:
        Alias for PARAMS.
    SCOPE:
        Only used for github device login to request scopes for the token
    """

    ETAG = "etag"
    HEADERS = "headers"
    METHOD = "method"
    PARAMS = "params"
    QUERY = "query"
    SCOPE = "scope"


class HttpStatusCode(IntEnum):
    """HTTP Status codes."""

    OK = 200
    CREATED = 201
    ACCEPTED = 202
    NON_AUTHORITATIVE = 203
    NO_CONTENT = 204
    MULTIPLE_CHOICES = 300
    MOVED_PERMANENTLY = 301
    FOUND = 302
    SEE_OTHER = 303
    NOT_MODIFIED = 304
    BAD_REQUEST = 400
    UNAUTHORIZED = 401
    RATELIMIT = 403
    FORBIDDEN = 403
    NOT_FOUND = 404
    TEAPOT = 418
    UNPROCESSABLE_ENTITY = 422
    INTERNAL_SERVER_ERROR = 500
    BAD_GATEWAY = 502
    SERVICE_UNAVAILABLE = 503
    GATEWAY_TIMEOUT = 504


class GitHubIssueLockReason(StrEnum):
    """Reason for issue lock."""

    OFF_TOPIC = "off-topic"
    SPAM = "spam"
    TOO_HEATED = "too heated"
    RESOLVED = "resolved"


class HttpMethod(StrEnum):
    """HTTP Methods."""

    GET = "GET"
    POST = "POST"
    PATCH = "PATCH"
    DELETE = "DELETE"
    PUT = "PUT"


class HttpContentType(StrEnum):
    """HTTP Content Types."""

    BASE_JSON = "application/json"
    BASE_ZIP = "application/zip"
    BASE_GZIP = "application/x-gzip"

    JSON = "application/json;charset=utf-8"
    TEXT_PLAIN = "text/plain;charset=utf-8"
    TEXT_HTML = "text/html;charset=utf-8"


class DeviceFlowError(StrEnum):
    """
    Errors for Device Flow.

    https://docs.github.com/en/developers/apps/authorizing-oauth-apps#error-codes-for-the-device-flow
    """

    ACCESS_DENIED = "access_denied"
    AUTHORIZATION_PENDING = "authorization_pending"
    EXPIRED_TOKEN = "expired_token"
    INCORRECT_CLIENT_CREDENTIALS = "incorrect_client_credentials"
    INCORRECT_DEVICE_CODE = "incorrect_device_code"
    SLOW_DOWN = "slow_down"
    UNSUPPORTED_GRANT_TYPE = "unsupported_grant_type"


HTTP_STATUS_CODE_GOOD_LIST: list[HttpStatusCode] = [
    HttpStatusCode.OK,
    HttpStatusCode.CREATED,
    HttpStatusCode.ACCEPTED,
    HttpStatusCode.NON_AUTHORITATIVE,
]


BASE_API_HEADERS: Dict[str, str] = {
    ACCEPT: GitHubRequestAcceptHeader.BASE.value,
    CONTENT_TYPE: HttpContentType.JSON.value,
    USER_AGENT: DEFAULT_USER_AGENT,
    HEADER_GITHUB_API_VERSION: DEFAULT_API_VERSION,
}



"""GitHub user models data class."""

from __future__ import annotations

from typing import Any, Dict

from .base import GitHubDataModelBase


class GitHubBaseUserModel(GitHubDataModelBase):
    """GitHub base user data class."""

    avatar_url: str | None = None
    events_url: str | None = None
    followers_url: str | None = None
    following_url: str | None = None
    gists_url: str | None = None
    gravatar_id: str | None = None
    html_url: str | None = None
    id: int | None = None
    login: str | None = None
    organizations_url: str | None = None
    received_events_url: str | None = None
    repos_url: str | None = None
    site_admin: bool | None = None
    starred_url: str | None = None
    subscriptions_url: str | None = None
    type: str | None = None
    url: str | None = None
    user_view_type: str | None = None


class GitHubUserModel(GitHubBaseUserModel):
    """GitHub user data class."""

    bio: str | None = None
    blog: str | None = None
    company: str | None = None
    created_at: str | None = None
    email: str | None = None
    followers: int | None = None
    following: int | None = None
    hireable: bool | None = None
    location: str | None = None
    name: str | None = None
    public_gists: int | None = None
    public_repos: int | None = None
    twitter_username: str | None = None
    updated_at: str | None = None


class GitHubAuthenticatedUserModel(GitHubUserModel):
    """GitHub authenticated user data class."""

    collaborators: int | None = None
    disk_usage: int | None = None
    owned_private_repos: int | None = None
    plan: GitHubUserPlanModel | None = None
    private_gists: int | None = None
    total_private_repos: int | None = None
    two_factor_authentication: bool | None = None

    def _generate_plan(self, data: Dict[str, Any] | None) -> GitHubUserPlanModel:
        """Generate GitHub user plan model."""
        return GitHubUserPlanModel(data) if data else None


class GitHubUserPlanModel(GitHubDataModelBase):
    """GitHub user plan data class."""

    collaborators: int | None = None
    name: str | None = None
    private_repos: int | None = None
    space: int | None = None



"""GitHub response data class."""

# pylint: disable=protected-access
from __future__ import annotations

from typing import Any, Dict, Generic

from aiohttp.client import ClientResponse
from yarl import URL

from ..const import GenericType, HttpStatusCode
from .base import GitHubDataModelBase


class GitHubResponseHeadersModel(GitHubDataModelBase):
    """GitHub response header model."""

    accept_ranges: str | None = None
    access_control_allow_origin: str | None = None
    access_control_expose_headers: str | None = None
    cache_control: str | None = None
    content_encoding: str | None = None
    content_security_policy: str | None = None
    content_length: str | None = None
    content_type: str | None = None
    date: str | None = None
    etag: str | None = None
    github_authentication_token_expiration: str | None = None
    last_modified: str | None = None
    link: str | None = None
    referrer_policy: str | None = None
    server: str | None = None
    strict_transport_security: str | None = None
    transfer_encoding: str | None = None
    vary: str | None = None
    retry_after: str | None = None
    expect_ct: str | None = None
    permissions_policy: str | None = None
    x_accepted_github_permissions: str | None = None
    x_accepted_oauth_scopes: str | None = None
    x_commonmarker_version: str | None = None
    x_content_type_options: str | None = None
    x_frame_options: str | None = None
    x_github_api_version_selected: str | None = None
    x_github_media_type: str | None = None
    x_github_request_id: str | None = None
    x_oauth_client_id: str | None = None
    x_oauth_scopes: str | None = None
    x_poll_interval: str | None = None
    x_ratelimit_limit: str | None = None
    x_ratelimit_remaining: str | None = None
    x_ratelimit_reset: str | None = None
    x_ratelimit_resource: str | None = None
    x_ratelimit_used: str | None = None
    x_xss_protection: str | None = None


class GitHubResponseModel(GitHubDataModelBase, Generic[GenericType]):
    """GitHub response data class."""

    _process_data: bool = False
    _raw_data: ClientResponse
    _slugify_keys: bool = True

    headers: GitHubResponseHeadersModel = None
    status: HttpStatusCode = None
    data: GenericType | None = None

    def __post_init__(self):
        """Post init."""
        self.headers = GitHubResponseHeadersModel(self._raw_data.headers)
        self.status = self._raw_data.status

    @property
    def etag(self) -> str | None:
        """Return the ETag for this response."""
        return self.headers.etag

    @property
    def pages(self) -> Dict[str, int]:
        """Return the pages for this response."""
        return {
            key: int(self._raw_data.links[key]["url"].query.get("page"))
            for key in self._raw_data.links
        }

    @property
    def is_last_page(self) -> bool:
        """Return True if this is the last page of results."""
        if self.headers.link is None:
            return True
        return self.page_number == self.pages.get("last", self.page_number)

    @property
    def page_number(self) -> int:
        """Return the current page number."""
        if self.headers.link is None:
            return 1
        if previous_page := self.pages.get("prev"):
            return previous_page + 1
        return 1

    @property
    def next_page_number(self) -> int | None:
        """Return the current page number."""
        if self.headers.link is None:
            return None
        return self.pages.get("next", None)

    @property
    def last_page_number(self) -> int | None:
        """Return the current page number."""
        if self.headers.link is None:
            return None
        return self.pages.get("last", None)



"""
Generated by generate/generate.py - 2020-09-08 18:45:20.581868
"""
import pytest


@pytest.fixture()
def oauth_fixtrue_response():
    return {
        "access_token": "e72e16c7e42f292c6912e7710c838347ae178b4a",
        "token_type": "bearer",
        "scope": "user",
    }



"""Test const."""

from aiogithubapi.const import StrEnum, IntEnum


def test_enum_value():
    """Test enum value."""

    class TestStrEnum(StrEnum):
        """Test StrEnum class"""

        TEST = "test"

    class TestIntEnum(IntEnum):
        """Test StrEnum class"""

        TEST = 4

    assert TestStrEnum.TEST == "test"
    assert TestStrEnum.TEST.value == "test"
    assert str(TestStrEnum.TEST) == "test"
    assert f"{TestStrEnum.TEST}" == "test"
    assert "{}".format(TestStrEnum.TEST) == "test"

    assert TestIntEnum.TEST == 4
    assert TestIntEnum.TEST.value == 4
    assert int(TestIntEnum.TEST) == 4



"""
Generated by generate/generate.py - 2020-08-02 12:25:07.157407
"""
from aiogithubapi.objects.repos.traffic.pageviews import (
    AIOGitHubAPIReposTrafficPageviews,
)

from tests.legacy.responses.repos.traffic.pageviews_fixtrue import (
    pageviews_fixtrue_response,
)


def test_pageviews(pageviews_fixtrue_response):
    obj = AIOGitHubAPIReposTrafficPageviews(pageviews_fixtrue_response)
    assert obj.count == pageviews_fixtrue_response["count"]
    assert obj.uniques == pageviews_fixtrue_response["uniques"]
    assert obj.views[0].timestamp == pageviews_fixtrue_response["views"][0]["timestamp"]
    assert obj.views[0].count == pageviews_fixtrue_response["views"][0]["count"]
    assert obj.views[0].uniques == pageviews_fixtrue_response["views"][0]["uniques"]



"""
Generated by generate/generate.py - 2020-08-02 15:48:55.441838
"""
from aiogithubapi.objects.repos.commit import AIOGitHubAPIReposCommit

from tests.legacy.responses.repos.commit_fixtrue import commit_fixtrue_response


def test_commit(commit_fixtrue_response):
    obj = AIOGitHubAPIReposCommit(commit_fixtrue_response)
    assert obj.url == commit_fixtrue_response["url"]
    assert obj.sha == commit_fixtrue_response["sha"]
    assert obj.node_id == commit_fixtrue_response["node_id"]
    assert obj.html_url == commit_fixtrue_response["html_url"]
    assert obj.comments_url == commit_fixtrue_response["comments_url"]
    assert obj.commit.url == commit_fixtrue_response["commit"]["url"]
    assert obj.commit.author.name == commit_fixtrue_response["commit"]["author"]["name"]
    assert obj.commit.author.email == commit_fixtrue_response["commit"]["author"]["email"]
    assert obj.commit.author.date == commit_fixtrue_response["commit"]["author"]["date"]
    assert obj.commit.committer.name == commit_fixtrue_response["commit"]["committer"]["name"]
    assert obj.commit.committer.email == commit_fixtrue_response["commit"]["committer"]["email"]
    assert obj.commit.committer.date == commit_fixtrue_response["commit"]["committer"]["date"]
    assert obj.commit.message == commit_fixtrue_response["commit"]["message"]
    assert obj.commit.tree.url == commit_fixtrue_response["commit"]["tree"]["url"]
    assert obj.commit.tree.sha == commit_fixtrue_response["commit"]["tree"]["sha"]
    assert obj.commit.comment_count == commit_fixtrue_response["commit"]["comment_count"]
    assert (
        obj.commit.verification.verified
        == commit_fixtrue_response["commit"]["verification"]["verified"]
    )
    assert (
        obj.commit.verification.reason
        == commit_fixtrue_response["commit"]["verification"]["reason"]
    )
    assert (
        obj.commit.verification.signature
        == commit_fixtrue_response["commit"]["verification"]["signature"]
    )
    assert (
        obj.commit.verification.payload
        == commit_fixtrue_response["commit"]["verification"]["payload"]
    )
    assert obj.author.login == commit_fixtrue_response["author"]["login"]
    assert obj.author.id == commit_fixtrue_response["author"]["id"]
    assert obj.author.node_id == commit_fixtrue_response["author"]["node_id"]
    assert obj.author.avatar_url == commit_fixtrue_response["author"]["avatar_url"]
    assert obj.author.gravatar_id == commit_fixtrue_response["author"]["gravatar_id"]
    assert obj.author.url == commit_fixtrue_response["author"]["url"]
    assert obj.author.html_url == commit_fixtrue_response["author"]["html_url"]
    assert obj.author.followers_url == commit_fixtrue_response["author"]["followers_url"]
    assert obj.author.following_url == commit_fixtrue_response["author"]["following_url"]
    assert obj.author.gists_url == commit_fixtrue_response["author"]["gists_url"]
    assert obj.author.starred_url == commit_fixtrue_response["author"]["starred_url"]
    assert obj.author.subscriptions_url == commit_fixtrue_response["author"]["subscriptions_url"]
    assert obj.author.organizations_url == commit_fixtrue_response["author"]["organizations_url"]
    assert obj.author.repos_url == commit_fixtrue_response["author"]["repos_url"]
    assert obj.author.events_url == commit_fixtrue_response["author"]["events_url"]
    assert (
        obj.author.received_events_url == commit_fixtrue_response["author"]["received_events_url"]
    )
    assert obj.author.type == commit_fixtrue_response["author"]["type"]
    assert obj.author.site_admin == commit_fixtrue_response["author"]["site_admin"]
    assert obj.committer.login == commit_fixtrue_response["committer"]["login"]
    assert obj.committer.id == commit_fixtrue_response["committer"]["id"]
    assert obj.committer.node_id == commit_fixtrue_response["committer"]["node_id"]
    assert obj.committer.avatar_url == commit_fixtrue_response["committer"]["avatar_url"]
    assert obj.committer.gravatar_id == commit_fixtrue_response["committer"]["gravatar_id"]
    assert obj.committer.url == commit_fixtrue_response["committer"]["url"]
    assert obj.committer.html_url == commit_fixtrue_response["committer"]["html_url"]
    assert obj.committer.followers_url == commit_fixtrue_response["committer"]["followers_url"]
    assert obj.committer.following_url == commit_fixtrue_response["committer"]["following_url"]
    assert obj.committer.gists_url == commit_fixtrue_response["committer"]["gists_url"]
    assert obj.committer.starred_url == commit_fixtrue_response["committer"]["starred_url"]
    assert (
        obj.committer.subscriptions_url == commit_fixtrue_response["committer"]["subscriptions_url"]
    )
    assert (
        obj.committer.organizations_url == commit_fixtrue_response["committer"]["organizations_url"]
    )
    assert obj.committer.repos_url == commit_fixtrue_response["committer"]["repos_url"]
    assert obj.committer.events_url == commit_fixtrue_response["committer"]["events_url"]
    assert (
        obj.committer.received_events_url
        == commit_fixtrue_response["committer"]["received_events_url"]
    )
    assert obj.committer.type == commit_fixtrue_response["committer"]["type"]
    assert obj.committer.site_admin == commit_fixtrue_response["committer"]["site_admin"]
    assert obj.parents[0].url == commit_fixtrue_response["parents"][0]["url"]
    assert obj.parents[0].sha == commit_fixtrue_response["parents"][0]["sha"]
    assert obj.stats.additions == commit_fixtrue_response["stats"]["additions"]
    assert obj.stats.deletions == commit_fixtrue_response["stats"]["deletions"]
    assert obj.stats.total == commit_fixtrue_response["stats"]["total"]
    assert obj.files[0].filename == commit_fixtrue_response["files"][0]["filename"]
    assert obj.files[0].additions == commit_fixtrue_response["files"][0]["additions"]
    assert obj.files[0].deletions == commit_fixtrue_response["files"][0]["deletions"]
    assert obj.files[0].changes == commit_fixtrue_response["files"][0]["changes"]
    assert obj.files[0].status == commit_fixtrue_response["files"][0]["status"]
    assert obj.files[0].raw_url == commit_fixtrue_response["files"][0]["raw_url"]
    assert obj.files[0].blob_url == commit_fixtrue_response["files"][0]["blob_url"]
    assert obj.files[0].patch == commit_fixtrue_response["files"][0]["patch"]



"""
Generated by generate/generate.py - 2020-08-02 10:38:59.444717
"""
import pytest


@pytest.fixture()
def fork_fixtrue_response():
    return {
        "id": 1296269,
        "node_id": "MDEwOlJlcG9zaXRvcnkxMjk2MjY5",
        "name": "Hello-World",
        "full_name": "octocat/Hello-World",
        "owner": {
            "login": "octocat",
            "id": 1,
            "node_id": "MDQ6VXNlcjE=",
            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octocat",
            "html_url": "https://github.com/octocat",
            "followers_url": "https://api.github.com/users/octocat/followers",
            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
            "organizations_url": "https://api.github.com/users/octocat/orgs",
            "repos_url": "https://api.github.com/users/octocat/repos",
            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octocat/received_events",
            "type": "User",
            "site_admin": False,
        },
        "private": False,
        "html_url": "https://github.com/octocat/Hello-World",
        "description": "This your first repo!",
        "fork": True,
        "url": "https://api.github.com/repos/octocat/Hello-World",
        "archive_url": "http://api.github.com/repos/octocat/Hello-World/{archive_format}{/ref}",
        "assignees_url": "http://api.github.com/repos/octocat/Hello-World/assignees{/user}",
        "blobs_url": "http://api.github.com/repos/octocat/Hello-World/git/blobs{/sha}",
        "branches_url": "http://api.github.com/repos/octocat/Hello-World/branches{/branch}",
        "collaborators_url": "http://api.github.com/repos/octocat/Hello-World/collaborators{/collaborator}",
        "comments_url": "http://api.github.com/repos/octocat/Hello-World/comments{/number}",
        "commits_url": "http://api.github.com/repos/octocat/Hello-World/commits{/sha}",
        "compare_url": "http://api.github.com/repos/octocat/Hello-World/compare/{base}...{head}",
        "contents_url": "http://api.github.com/repos/octocat/Hello-World/contents/{+path}",
        "contributors_url": "http://api.github.com/repos/octocat/Hello-World/contributors",
        "deployments_url": "http://api.github.com/repos/octocat/Hello-World/deployments",
        "downloads_url": "http://api.github.com/repos/octocat/Hello-World/downloads",
        "events_url": "http://api.github.com/repos/octocat/Hello-World/events",
        "forks_url": "http://api.github.com/repos/octocat/Hello-World/forks",
        "git_commits_url": "http://api.github.com/repos/octocat/Hello-World/git/commits{/sha}",
        "git_refs_url": "http://api.github.com/repos/octocat/Hello-World/git/refs{/sha}",
        "git_tags_url": "http://api.github.com/repos/octocat/Hello-World/git/tags{/sha}",
        "git_url": "git:github.com/octocat/Hello-World.git",
        "issue_comment_url": "http://api.github.com/repos/octocat/Hello-World/issues/comments{/number}",
        "issue_events_url": "http://api.github.com/repos/octocat/Hello-World/issues/events{/number}",
        "issues_url": "http://api.github.com/repos/octocat/Hello-World/issues{/number}",
        "keys_url": "http://api.github.com/repos/octocat/Hello-World/keys{/key_id}",
        "labels_url": "http://api.github.com/repos/octocat/Hello-World/labels{/name}",
        "languages_url": "http://api.github.com/repos/octocat/Hello-World/languages",
        "merges_url": "http://api.github.com/repos/octocat/Hello-World/merges",
        "milestones_url": "http://api.github.com/repos/octocat/Hello-World/milestones{/number}",
        "notifications_url": "http://api.github.com/repos/octocat/Hello-World/notifications{?since,all,participating}",
        "pulls_url": "http://api.github.com/repos/octocat/Hello-World/pulls{/number}",
        "releases_url": "http://api.github.com/repos/octocat/Hello-World/releases{/id}",
        "ssh_url": "git@github.com:octocat/Hello-World.git",
        "stargazers_url": "http://api.github.com/repos/octocat/Hello-World/stargazers",
        "statuses_url": "http://api.github.com/repos/octocat/Hello-World/statuses/{sha}",
        "subscribers_url": "http://api.github.com/repos/octocat/Hello-World/subscribers",
        "subscription_url": "http://api.github.com/repos/octocat/Hello-World/subscription",
        "tags_url": "http://api.github.com/repos/octocat/Hello-World/tags",
        "teams_url": "http://api.github.com/repos/octocat/Hello-World/teams",
        "trees_url": "http://api.github.com/repos/octocat/Hello-World/git/trees{/sha}",
        "clone_url": "https://github.com/octocat/Hello-World.git",
        "mirror_url": "git:git.example.com/octocat/Hello-World",
        "hooks_url": "http://api.github.com/repos/octocat/Hello-World/hooks",
        "svn_url": "https://svn.github.com/octocat/Hello-World",
        "homepage": "https://github.com",
        "language": None,
        "forks_count": 9,
        "stargazers_count": 80,
        "watchers_count": 80,
        "size": 108,
        "default_branch": "main",
        "open_issues_count": 0,
        "is_template": True,
        "topics": ["octocat", "atom", "electron", "api"],
        "has_issues": True,
        "has_projects": True,
        "has_wiki": True,
        "has_pages": False,
        "has_downloads": True,
        "archived": False,
        "disabled": False,
        "visibility": "public",
        "pushed_at": "2011-01-26T19:06:43Z",
        "created_at": "2011-01-26T19:01:12Z",
        "updated_at": "2011-01-26T19:14:43Z",
        "permissions": {"admin": False, "push": False, "pull": True},
        "template_repository": None,
        "temp_clone_token": "ABTLWHOULUVAXGTRYU7OC2876QJ2O",
        "delete_branch_on_merge": True,
        "subscribers_count": 42,
        "network_count": 0,
        "license": {
            "key": "mit",
            "name": "MIT License",
            "spdx_id": "MIT",
            "url": "https://api.github.com/licenses/mit",
            "node_id": "MDc6TGljZW5zZW1pdA==",
        },
    }



"""
AIOGitHubAPI: Repository Release

https://developer.github.com/v3/repos/releases/
"""

# pylint: disable=missing-docstring
from datetime import datetime

from ...objects.base import AIOGitHubAPIBase
from ...objects.repository.content import AIOGitHubAPIRepositoryContent


class AIOGitHubAPIRepositoryRelease(AIOGitHubAPIBase):
    """Repository Release GitHub API implementation."""

    @property
    def tag_name(self):
        return self.attributes.get("tag_name")

    @property
    def name(self):
        return self.attributes.get("name")

    @property
    def published_at(self):
        return datetime.strptime(self.attributes.get("published_at"), "%Y-%m-%dT%H:%M:%SZ")

    @property
    def draft(self):
        return self.attributes.get("draft")

    @property
    def prerelease(self):
        return self.attributes.get("prerelease")

    @property
    def assets(self):
        return [AIOGitHubAPIRepositoryContent(x) for x in self.attributes.get("assets", [])]



"""GitHub tag data class."""

from __future__ import annotations

from .base import GitHubDataModelBase


class _Commit(GitHubDataModelBase):
    """Representation of a GitHub tag commit."""

    sha: str | None = None
    url: str | None = None


class GitHubTagModel(GitHubDataModelBase):
    """GitHub tag data class."""

    name: str | None = None
    commit: _Commit | None = None
    zipball_url: str | None = None
    tarball_url: str | None = None

    def _generate_commit(self, data: dict) -> _Commit:
        """Generate commit data."""
        return _Commit(data)



"""
Class object for AIOGitHubAPIReposFork
Documentation: https://docs.github.com/en/rest/reference/repos#list-forks

Generated by generate/generate.py - 2020-08-02 10:38:59.444717
"""

from ..base import AIOGitHubAPIBase


class Owner(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def followers_url(self):
        return self.attributes.get("followers_url", "")

    @property
    def following_url(self):
        return self.attributes.get("following_url", "")

    @property
    def gists_url(self):
        return self.attributes.get("gists_url", "")

    @property
    def starred_url(self):
        return self.attributes.get("starred_url", "")

    @property
    def subscriptions_url(self):
        return self.attributes.get("subscriptions_url", "")

    @property
    def organizations_url(self):
        return self.attributes.get("organizations_url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def received_events_url(self):
        return self.attributes.get("received_events_url", "")

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin", False)


class Permissions(AIOGitHubAPIBase):
    @property
    def admin(self):
        return self.attributes.get("admin", False)

    @property
    def push(self):
        return self.attributes.get("push", False)

    @property
    def pull(self):
        return self.attributes.get("pull", True)


class License(AIOGitHubAPIBase):
    @property
    def key(self):
        return self.attributes.get("key", "")

    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def spdx_id(self):
        return self.attributes.get("spdx_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")


class AIOGitHubAPIReposFork(AIOGitHubAPIBase):
    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def full_name(self):
        return self.attributes.get("full_name", "")

    @property
    def owner(self):
        return Owner(self.attributes.get("owner", {}))

    @property
    def private(self):
        return self.attributes.get("private", False)

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def description(self):
        return self.attributes.get("description", "")

    @property
    def fork(self):
        return self.attributes.get("fork", True)

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def archive_url(self):
        return self.attributes.get("archive_url", "")

    @property
    def assignees_url(self):
        return self.attributes.get("assignees_url", "")

    @property
    def blobs_url(self):
        return self.attributes.get("blobs_url", "")

    @property
    def branches_url(self):
        return self.attributes.get("branches_url", "")

    @property
    def collaborators_url(self):
        return self.attributes.get("collaborators_url", "")

    @property
    def comments_url(self):
        return self.attributes.get("comments_url", "")

    @property
    def commits_url(self):
        return self.attributes.get("commits_url", "")

    @property
    def compare_url(self):
        return self.attributes.get("compare_url", "")

    @property
    def contents_url(self):
        return self.attributes.get("contents_url", "")

    @property
    def contributors_url(self):
        return self.attributes.get("contributors_url", "")

    @property
    def deployments_url(self):
        return self.attributes.get("deployments_url", "")

    @property
    def downloads_url(self):
        return self.attributes.get("downloads_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def forks_url(self):
        return self.attributes.get("forks_url", "")

    @property
    def git_commits_url(self):
        return self.attributes.get("git_commits_url", "")

    @property
    def git_refs_url(self):
        return self.attributes.get("git_refs_url", "")

    @property
    def git_tags_url(self):
        return self.attributes.get("git_tags_url", "")

    @property
    def git_url(self):
        return self.attributes.get("git_url", "")

    @property
    def issue_comment_url(self):
        return self.attributes.get("issue_comment_url", "")

    @property
    def issue_events_url(self):
        return self.attributes.get("issue_events_url", "")

    @property
    def issues_url(self):
        return self.attributes.get("issues_url", "")

    @property
    def keys_url(self):
        return self.attributes.get("keys_url", "")

    @property
    def labels_url(self):
        return self.attributes.get("labels_url", "")

    @property
    def languages_url(self):
        return self.attributes.get("languages_url", "")

    @property
    def merges_url(self):
        return self.attributes.get("merges_url", "")

    @property
    def milestones_url(self):
        return self.attributes.get("milestones_url", "")

    @property
    def notifications_url(self):
        return self.attributes.get("notifications_url", "")

    @property
    def pulls_url(self):
        return self.attributes.get("pulls_url", "")

    @property
    def releases_url(self):
        return self.attributes.get("releases_url", "")

    @property
    def ssh_url(self):
        return self.attributes.get("ssh_url", "")

    @property
    def stargazers_url(self):
        return self.attributes.get("stargazers_url", "")

    @property
    def statuses_url(self):
        return self.attributes.get("statuses_url", "")

    @property
    def subscribers_url(self):
        return self.attributes.get("subscribers_url", "")

    @property
    def subscription_url(self):
        return self.attributes.get("subscription_url", "")

    @property
    def tags_url(self):
        return self.attributes.get("tags_url", "")

    @property
    def teams_url(self):
        return self.attributes.get("teams_url", "")

    @property
    def trees_url(self):
        return self.attributes.get("trees_url", "")

    @property
    def clone_url(self):
        return self.attributes.get("clone_url", "")

    @property
    def mirror_url(self):
        return self.attributes.get("mirror_url", "")

    @property
    def hooks_url(self):
        return self.attributes.get("hooks_url", "")

    @property
    def svn_url(self):
        return self.attributes.get("svn_url", "")

    @property
    def homepage(self):
        return self.attributes.get("homepage", "")

    @property
    def language(self):
        return self.attributes.get("language", None)

    @property
    def forks_count(self):
        return self.attributes.get("forks_count", None)

    @property
    def stargazers_count(self):
        return self.attributes.get("stargazers_count", None)

    @property
    def watchers_count(self):
        return self.attributes.get("watchers_count", None)

    @property
    def size(self):
        return self.attributes.get("size", None)

    @property
    def default_branch(self):
        return self.attributes.get("default_branch", "")

    @property
    def open_issues_count(self):
        return self.attributes.get("open_issues_count", None)

    @property
    def is_template(self):
        return self.attributes.get("is_template", True)

    @property
    def topics(self):
        return self.attributes.get("topics", [])

    @property
    def has_issues(self):
        return self.attributes.get("has_issues", True)

    @property
    def has_projects(self):
        return self.attributes.get("has_projects", True)

    @property
    def has_wiki(self):
        return self.attributes.get("has_wiki", True)

    @property
    def has_pages(self):
        return self.attributes.get("has_pages", False)

    @property
    def has_downloads(self):
        return self.attributes.get("has_downloads", True)

    @property
    def archived(self):
        return self.attributes.get("archived", False)

    @property
    def disabled(self):
        return self.attributes.get("disabled", False)

    @property
    def visibility(self):
        return self.attributes.get("visibility", "")

    @property
    def pushed_at(self):
        return self.attributes.get("pushed_at", "")

    @property
    def created_at(self):
        return self.attributes.get("created_at", "")

    @property
    def updated_at(self):
        return self.attributes.get("updated_at", "")

    @property
    def permissions(self):
        return Permissions(self.attributes.get("permissions", {}))

    @property
    def template_repository(self):
        return self.attributes.get("template_repository", None)

    @property
    def temp_clone_token(self):
        return self.attributes.get("temp_clone_token", "")

    @property
    def delete_branch_on_merge(self):
        return self.attributes.get("delete_branch_on_merge", True)

    @property
    def subscribers_count(self):
        return self.attributes.get("subscribers_count", None)

    @property
    def network_count(self):
        return self.attributes.get("network_count", None)

    @property
    def license(self):
        return License(self.attributes.get("license", {}))



"""Test users namespace."""
# pylint: disable=missing-docstring
import pytest

from aiogithubapi import GitHubAPI

from tests.common import TEST_USER_NAME, MockedRequests


@pytest.mark.asyncio
async def test_get_user(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.users.get(TEST_USER_NAME)
    assert response.status == 200
    assert response.data.name == "The Octocat"
    assert mock_requests.called == 1
    assert mock_requests.last_request["url"] == "https://api.github.com/users/octocat"


@pytest.mark.asyncio
async def test_get_user_starred(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.users.starred(TEST_USER_NAME)
    assert response.status == 200
    first_repo = response.data[0]
    assert first_repo.name == "Hello-World"
    assert mock_requests.called == 1
    assert mock_requests.last_request["url"] == "https://api.github.com/users/octocat/starred"


@pytest.mark.asyncio
async def test_get_user_repos(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.users.repos(TEST_USER_NAME)
    assert response.status == 200
    first_repo = response.data[0]
    assert first_repo.name == "Hello-World"
    assert mock_requests.called == 1
    assert mock_requests.last_request["url"] == "https://api.github.com/users/octocat/repos"


@pytest.mark.asyncio
async def test_get_user_orgs(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.users.orgs(TEST_USER_NAME)
    assert response.status == 200
    first_repo = response.data[0]
    assert first_repo.login == "github"
    assert mock_requests.called == 1
    assert mock_requests.last_request["url"] == "https://api.github.com/users/octocat/orgs"



"""Fixtures."""
# pylint: disable=missing-docstring
import pytest


@pytest.fixture()
def tree_response():
    return {
        "sha": "fc6274d15fa3ae2ab983129fb037999f264ba9a7",
        "url": "https://api.github.com/repos/octocat/Hello-World/trees/fc6274d15fa3ae2ab983129fb037999f264ba9a7",
        "tree": [
            {
                "path": "subdir/file.txt",
                "mode": "100644",
                "type": "blob",
                "size": 132,
                "sha": "7c258a9869f33c1e1e1f74fbb32f07c86cb5a75b",
                "url": "https://api.github.com/repos/octocat/Hello-World/git/7c258a9869f33c1e1e1f74fbb32f07c86cb5a75b",
            },
            {
                "path": "subdir",
                "mode": "100644",
                "type": "tree",
                "size": 132,
                "sha": "7c258a9869f33c1e1e1f74fbb32f07c86cb5a75b",
                "url": "https://api.github.com/repos/octocat/Hello-World/git/7c258a9869f33c1e1e1f74fbb32f07c86cb5a75b",
            },
        ],
        "truncated": False,
    }



"""GitHub notification models"""

from __future__ import annotations

from typing import Any, Dict

from .base import GitHubDataModelBase
from .repository import GitHubRepositoryModel


class _Subject(GitHubDataModelBase):
    """Notification subject."""

    title: str | None = None
    url: str | None = None
    latest_comment_url: str | None = None
    type: str | None = None


class GitHubNotificationModel(GitHubDataModelBase):
    """
    GitHub notification data class.

    https://docs.github.com/en/rest/activity/notifications
    """

    id: str | None = None
    repository: GitHubRepositoryModel | None = None
    subject: _Subject | None = None
    reason: str | None = None
    unread: bool | None = None
    updated_at: str | None = None
    last_read_at: str | None = None
    url: str | None = None
    subscription_url: str | None = None

    def _generate_repository(self, data: Dict[str, Any] | None) -> GitHubRepositoryModel:
        """Generate repository data."""
        return GitHubRepositoryModel(data) if data else None

    def _generate_subject(self, data: Dict[str, Any] | None) -> _Subject:
        """Generate subject data."""
        return _Subject(data) if data else None



"""Test releases namespace."""
# pylint: disable=missing-docstring
import pytest

from aiogithubapi import GitHubAPI, GitHubReleaseModel

from tests.common import TEST_REPOSITORY_NAME, MockedRequests


@pytest.mark.asyncio
async def test_list_releases(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.repos.releases.list(TEST_REPOSITORY_NAME)
    assert response.status == 200
    assert isinstance(response.data, list)
    assert isinstance(response.data[0], GitHubReleaseModel)
    assert response.data[0].name == "First release"
    assert mock_requests.called == 1
    assert (
        mock_requests.last_request["url"]
        == "https://api.github.com/repos/octocat/hello-world/releases"
    )

@pytest.mark.asyncio
async def test_latest(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.repos.releases.latest(TEST_REPOSITORY_NAME)
    assert response.status == 200
    assert isinstance(response.data, GitHubReleaseModel)
    assert response.data.name == "First release"
    assert mock_requests.called == 1
    assert (
        mock_requests.last_request["url"]
        == "https://api.github.com/repos/octocat/hello-world/releases/latest"
    )


"""GitHub organization model."""

from __future__ import annotations

from typing import Any, Dict

from .base import GitHubDataModelBase


class _Plan(GitHubDataModelBase):
    """GitHub organization plan data model."""

    name: str | None = None
    space: int | None = None
    private_repos: int | None = None
    filled_seats: int | None = None
    seats: int | None = None


class GitHubOrganizationModel(GitHubDataModelBase):
    """GitHub organization model."""

    login: str | None = None
    id: int | None = None
    node_id: str | None = None
    url: str | None = None
    repos_url: str | None = None
    events_url: str | None = None
    hooks_url: str | None = None
    issues_url: str | None = None
    members_url: str | None = None
    public_members_url: str | None = None
    avatar_url: str | None = None
    description: str | None = None
    name: str | None = None
    company: str | None = None
    blog: str | None = None
    location: str | None = None
    email: str | None = None
    twitter_username: str | None = None
    is_verified: bool | None = None
    has_organization_projects: bool | None = None
    has_repository_projects: bool | None = None
    public_repos: int | None = None
    public_gists: int | None = None
    followers: int | None = None
    following: int | None = None
    html_url: str | None = None
    created_at: str | None = None
    updated_at: str | None = None
    type: str | None = None
    total_private_repos: int | None = None
    owned_private_repos: int | None = None
    private_gists: int | None = None
    disk_usage: int | None = None
    collaborators: int | None = None
    billing_email: str | None = None
    plan: _Plan | None = None
    default_repository_permission: str | None = None
    members_can_create_repositories: bool | None = None
    two_factor_requirement_enabled: bool | None = None
    members_allowed_repository_creation_type: str | None = None
    members_can_create_public_repositories: bool | None = None
    members_can_create_private_repositories: bool | None = None
    members_can_create_internal_repositories: bool | None = None
    members_can_create_pages: bool | None = None
    members_can_fork_private_repositories: bool | None = None

    def _generate_plan(self, data: Dict[str, Any]) -> _Plan:
        """Generate GitHubLabelModel list from data."""
        return _Plan(data) if data else None


class GitHubOrganizationMinimalModel(GitHubDataModelBase):
    """GitHub organization minimal model."""

    login: str | None = None
    id: int | None = None
    node_id: str | None = None
    url: str | None = None
    repos_url: str | None = None
    events_url: str | None = None
    hooks_url: str | None = None
    issues_url: str | None = None
    members_url: str | None = None
    public_members_url: str | None = None
    avatar_url: str | None = None
    description: str | None = None



"""
Class for OAuth device flow authentication.

https://docs.github.com/en/developers/apps/authorizing-oauth-apps#device-flow
"""

from __future__ import annotations

import asyncio
from datetime import datetime
from typing import Any, Dict

import aiohttp

from .client import GitHubClient
from .const import (
    BASE_GITHUB_URL,
    OAUTH_ACCESS_TOKEN_PATH,
    OAUTH_DEVICE_LOGIN_PATH,
    DeviceFlowError,
    GitHubClientKwarg,
    GitHubRequestKwarg,
    HttpMethod,
)
from .exceptions import GitHubAuthenticationException, GitHubException
from .legacy.device import AIOGitHubAPIDeviceLogin as LegacyAIOGitHubAPIDeviceLogin
from .models.base import GitHubBase
from .models.device_login import GitHubLoginDeviceModel
from .models.login_oauth import GitHubLoginOauthModel
from .models.response import GitHubResponseModel


class AIOGitHubAPIDeviceLogin(LegacyAIOGitHubAPIDeviceLogin):
    """Dummy class to not break existing code."""


class GitHubDeviceAPI(GitHubBase):
    """GitHub API OAuth device flow"""

    _close_session = False

    def __init__(
        self,
        client_id: str,
        session: aiohttp.ClientSession | None = None,
        **kwargs: Dict[GitHubClientKwarg, Any],
    ):
        """
        Initialises a GitHub API OAuth device flow.

        **Arguments**:

        `client_id` (Optional)

        The client ID of your OAuth app.


        `session` (Optional)

        `aiohttp.ClientSession` to be used by this package.
        If you do not pass one, one will be created for you.

        `**kwargs` (Optional)

        Pass additional arguments.
        See the `aiogithubapi.const.GitHubClientKwarg` enum for valid options.

        https://docs.github.com/en/developers/apps/authorizing-oauth-apps#device-flow
        """
        self.client_id = client_id
        self._interval = 5
        self._expires = None

        if session is None:
            session = aiohttp.ClientSession()
            self._close_session = True

        self._session = session

        if GitHubClientKwarg.BASE_URL not in kwargs:
            kwargs[GitHubClientKwarg.BASE_URL] = BASE_GITHUB_URL

        self._client = GitHubClient(session=session, **kwargs)

    async def __aenter__(self) -> GitHubDeviceAPI:
        """Async enter."""
        return self

    async def __aexit__(self, *exc_info) -> None:
        """Async exit."""
        await self.close_session()

    async def close_session(self) -> None:
        """Close open client session."""
        if self._session and self._close_session:
            await self._session.close()

    async def register(
        self,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubLoginDeviceModel]:
        """Register the device and return an object that contains the user code for authorization."""
        response = await self._client.async_call_api(
            endpoint=OAUTH_DEVICE_LOGIN_PATH,
            **{
                **kwargs,
                GitHubRequestKwarg.METHOD: HttpMethod.POST,
                GitHubRequestKwarg.PARAMS: {
                    "client_id": self.client_id,
                    "scope": kwargs.get(GitHubRequestKwarg.SCOPE, ""),
                },
            },
        )
        response.data = GitHubLoginDeviceModel(response.data)
        self._interval = response.data.interval
        self._expires = datetime.timestamp(datetime.now()) + response.data.expires_in
        return response

    async def activation(
        self,
        device_code: str,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubLoginOauthModel]:
        """
        Wait for the user to enter the code and activate the device.

        **Arguments**:

        `device_code`

        The device_code that was returned when registering the device.

        """
        if self._expires is None:
            raise GitHubException("Expiration has passed, re-run the registration")

        _user_confirmed = None
        while _user_confirmed is None:
            if self._expires < datetime.timestamp(datetime.now()):
                raise GitHubException("User took too long to enter key")

            response = await self._client.async_call_api(
                endpoint=OAUTH_ACCESS_TOKEN_PATH,
                **{
                    **kwargs,
                    GitHubRequestKwarg.METHOD: HttpMethod.POST,
                    GitHubRequestKwarg.PARAMS: {
                        "client_id": self.client_id,
                        "device_code": device_code,
                        "grant_type": "urn:ietf:params:oauth:grant-type:device_code",
                    },
                },
            )

            if (
                isinstance(response.data, str)
                and "You have exceeded a secondary rate limit" in response.data
            ):
                raise GitHubAuthenticationException(
                    "Secondary rate limit exceeded, try again in 1 hour"
                )

            if error := response.data.get("error"):
                self.logger.debug(response.data.get("error_description"))
                if error in (DeviceFlowError.AUTHORIZATION_PENDING, DeviceFlowError.SLOW_DOWN):
                    if interval := response.data.get("interval"):
                        self._interval = interval
                        self.logger.info(
                            "Got new interval instruction of %s from the API", interval
                        )
                    await asyncio.sleep(self._interval or 5)
                else:
                    raise GitHubException(response.data.get("error_description"))
            else:
                response.data = GitHubLoginOauthModel(response.data)
                break

        return response



# pylint: disable=missing-docstring, redefined-outer-name, unused-import
import datetime
import json

import pytest

from aiogithubapi import AIOGitHubAPIException, AIOGitHubAPINotModifiedException, GitHub

from tests.common import TOKEN
from tests.legacy.responses.base import base_response
from tests.legacy.responses.branch import branch_response
from tests.legacy.responses.contents import (
    contents_file_response,
    contents_list_response,
)
from tests.legacy.responses.issue_fixture import issue_response
from tests.legacy.responses.releases import releases_response
from tests.legacy.responses.repository_fixture import repository_response
from tests.legacy.responses.tree import tree_response


@pytest.mark.asyncio
async def test_get_repository(aresponses, repository_response):
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World",
        "get",
        aresponses.Response(
            text=json.dumps(repository_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    async with GitHub(TOKEN) as github:
        repository = await github.get_repo("octocat/Hello-World")
        assert repository.description == "This your first repo!"
        assert repository.id == 1296269
        assert repository.name == "Hello-World"
        assert repository.full_name == "octocat/Hello-World"
        assert repository.pushed_at == datetime.datetime(2011, 1, 26, 19, 6, 43)
        assert not repository.archived
        assert repository.topics == ["octocat", "atom", "electron", "api"]
        assert not repository.fork
        assert repository.forks_count == 9
        assert repository.default_branch == "main"
        assert repository.last_commit is None
        assert repository.homepage == "https://github.com"
        assert repository.stargazers_count == 80
        assert repository.watchers_count == 80
        assert repository.owner.login == "octocat"


@pytest.mark.asyncio
async def test_set_last_commit(aresponses, repository_response, branch_response):
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World",
        "get",
        aresponses.Response(
            text=json.dumps(repository_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/branches/main",
        "get",
        aresponses.Response(
            text=json.dumps(branch_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/branches/main",
        "get",
        aresponses.Response(status=304),
    )
    async with GitHub(TOKEN) as github:
        repository = await github.get_repo("octocat/Hello-World")
        assert repository.last_commit is None
        await repository.set_last_commit()
        assert repository.last_commit == "7fd1a60"

        with pytest.raises(AIOGitHubAPINotModifiedException):
            await repository.set_last_commit(github.client.last_response.etag)


@pytest.mark.asyncio
async def test_get_contents_file(aresponses, repository_response, contents_file_response):
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World",
        "get",
        aresponses.Response(
            text=json.dumps(repository_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/contents/README.md",
        "get",
        aresponses.Response(
            text=json.dumps(contents_file_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    async with GitHub(TOKEN) as github:
        repository = await github.get_repo("octocat/Hello-World")
        contents = await repository.get_contents("README.md", "main")
        assert contents.name == "README.md"


@pytest.mark.asyncio
async def test_get_contents_list(aresponses, repository_response, contents_list_response):
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World",
        "get",
        aresponses.Response(
            text=json.dumps(repository_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/contents/",
        "get",
        aresponses.Response(
            text=json.dumps(contents_list_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    async with GitHub(TOKEN) as github:
        repository = await github.get_repo("octocat/Hello-World")
        contents = await repository.get_contents("", "main")
        assert len(contents) == 2


@pytest.mark.asyncio
async def test_get_tree(aresponses, repository_response, tree_response):
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World",
        "get",
        aresponses.Response(
            text=json.dumps(repository_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/git/trees/main",
        "get",
        aresponses.Response(
            text=json.dumps(tree_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    async with GitHub(TOKEN) as github:
        repository = await github.get_repo("octocat/Hello-World")
        contents = await repository.get_tree("main")
        assert contents[0].full_path == "subdir/file.txt"
        with pytest.raises(AIOGitHubAPIException):
            await repository.get_tree()


@pytest.mark.asyncio
async def test_get_rendered_contents(aresponses, repository_response):
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World",
        "get",
        aresponses.Response(
            text=json.dumps(repository_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/contents/README.md",
        "get",
        aresponses.Response(
            text="test",
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/contents/README.md",
        "get",
        aresponses.Response(status=304),
    )
    async with GitHub(TOKEN) as github:
        repository = await github.get_repo("octocat/Hello-World")
        rendered = await repository.get_rendered_contents("README.md", "main")
        assert rendered == "test"

        with pytest.raises(AIOGitHubAPINotModifiedException):
            await repository.get_rendered_contents(
                "README.md", "main", etag=github.client.last_response.etag
            )


@pytest.mark.asyncio
async def test_get_releases(aresponses, repository_response, releases_response):
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World",
        "get",
        aresponses.Response(
            text=json.dumps(repository_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/releases",
        "get",
        aresponses.Response(
            text=json.dumps(releases_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/releases",
        "get",
        aresponses.Response(
            text=json.dumps(releases_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/releases",
        "get",
        aresponses.Response(
            text=json.dumps(releases_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/releases",
        "get",
        aresponses.Response(status=304),
    )
    async with GitHub(TOKEN) as github:
        repository = await github.get_repo("octocat/Hello-World")
        releases = await repository.get_releases()
        assert len(releases) == 1
        releases = await repository.get_releases(prerelease=True)
        assert len(releases) == 2
        assert releases[0].tag_name == "v1.0.0"
        assert not releases[0].prerelease
        releases = await repository.get_releases(prerelease=True, returnlimit=1)
        assert len(releases) == 1

        with pytest.raises(AIOGitHubAPINotModifiedException):
            await repository.get_releases(etag=github.client.last_response.as_dict()["etag"])


@pytest.mark.asyncio
async def test_get_issue(aresponses, repository_response, issue_response):
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World",
        "get",
        aresponses.Response(
            text=json.dumps(repository_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/issues/1",
        "get",
        aresponses.Response(
            text=json.dumps(issue_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/issues/1",
        "get",
        aresponses.Response(status=304),
    )
    async with GitHub(TOKEN) as github:
        repository = await github.get_repo("octocat/Hello-World")
        issue = await repository.get_issue(1)
        assert issue.title == "Found a bug"

        with pytest.raises(AIOGitHubAPINotModifiedException):
            await repository.get_issue(1, etag=github.client.last_response.etag)


@pytest.mark.asyncio
async def test_create_issue(aresponses, repository_response):
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World",
        "get",
        aresponses.Response(
            text=json.dumps(repository_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/issues",
        "post",
        aresponses.Response(
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    async with GitHub(TOKEN) as github:
        repository = await github.get_repo("octocat/Hello-World")
        data = {
            "title": "test",
            "body": "body",
            "state": "closed",
            "milestone": "v1.0",
            "labels": ["test"],
            "assignees": ["octocat"],
        }
        await repository.create_issue(**data)


@pytest.mark.asyncio
async def test_get_last_commit(aresponses, repository_response, branch_response):
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World",
        "get",
        aresponses.Response(
            text=json.dumps(repository_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/branches/main",
        "get",
        aresponses.Response(
            text=json.dumps(branch_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/branches/main",
        "get",
        aresponses.Response(status=304),
    )
    async with GitHub(TOKEN) as github:
        repository = await github.get_repo("octocat/Hello-World")
        assert repository.last_commit is None
        commit = await repository.get_last_commit()
        assert commit.sha == "7fd1a60b01f91b314f59955a4e4d4e80d8edf11d"

        with pytest.raises(AIOGitHubAPINotModifiedException):
            await repository.get_last_commit(etag=github.client.last_response.etag)
